# -*- generated by 1.0.2 -*-
import da
PatternExpr_238 = da.pat.TuplePattern([da.pat.ConstantPattern('ack'), da.pat.FreePattern('c2'), da.pat.BoundPattern('_BoundPattern242_')])
PatternExpr_281 = da.pat.TuplePattern([da.pat.ConstantPattern('request'), da.pat.FreePattern('c2'), da.pat.FreePattern('p')])
PatternExpr_306 = da.pat.TuplePattern([da.pat.ConstantPattern('release'), da.pat.FreePattern(None), da.pat.FreePattern('p')])
PatternExpr_370 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_375 = da.pat.BoundPattern('_BoundPattern377_')
PatternExpr_378 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern384_')]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_470 = da.pat.TuplePattern([da.pat.ConstantPattern('done'), da.pat.BoundPattern('_BoundPattern473_')])
PatternExpr_476 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done'), da.pat.BoundPattern('_BoundPattern486_')])])
_config_object = {}
import sys
da.import_da('da.lib.laclock')
from da.lib.laclock import LamportClock

class P(LamportClock, da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._PReceivedEvent_0 = []
        self._PReceivedEvent_3 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_0', PatternExpr_238, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_1', PatternExpr_281, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_280]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_2', PatternExpr_306, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_305]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_3', PatternExpr_370, sources=[PatternExpr_375], destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, s, nrequests, **rest_494):
        super().setup(s=s, nrequests=nrequests, **rest_494)
        self._state.s = s
        self._state.nrequests = nrequests
        self._state.q = set()

    def run(self):
        self.output('My name is', self.nameof(self._id))

        def task():
            self.output('in cs')
            self.output('release cs')
        for i in range(self._state.nrequests):
            self.mutex(task)
        self.send(('done', self._id), self.parent())
        super()._label('_st_label_367', block=False)
        _st_label_367 = 0
        while (_st_label_367 == 0):
            _st_label_367 += 1
            if PatternExpr_378.match_iter(self._PReceivedEvent_3, _BoundPattern384_=self.parent(), SELF_ID=self._id):
                _st_label_367 += 1
            else:
                super()._label('_st_label_367', block=True)
                _st_label_367 -= 1
        self.output('terminating')

    def mutex(self, task):
        super()._label('request', block=False)
        c = self.logical_clock()
        self.send(('request', c, self._id), self._state.s)
        self._state.q.add(('request', c, self._id))
        super()._label('_st_label_203', block=False)
        p = c2 = None

        def UniversalOpExpr_205():
            nonlocal p, c2
            for (_ConstantPattern209_, c2, p) in self._state.q:
                if (_ConstantPattern209_ == 'request'):
                    if (not (((c2, p) == (c, self._id)) or ((c, self._id) < (c2, p)))):
                        return False
            return True
        p = c2 = None

        def UniversalOpExpr_231():
            nonlocal p, c2
            for p in self._state.s:

                def ExistentialOpExpr_236(p):
                    nonlocal c2
                    for (_, _, (_ConstantPattern253_, c2, _BoundPattern256_)) in self._PReceivedEvent_0:
                        if (_ConstantPattern253_ == 'ack'):
                            if (_BoundPattern256_ == p):
                                if (c2 > c):
                                    return True
                    return False
                if (not ExistentialOpExpr_236(p=p)):
                    return False
            return True
        _st_label_203 = 0
        while (_st_label_203 == 0):
            _st_label_203 += 1
            if (UniversalOpExpr_205() and UniversalOpExpr_231()):
                _st_label_203 += 1
            else:
                super()._label('_st_label_203', block=True)
                _st_label_203 -= 1
        super()._label('critical_section', block=False)
        task()
        super()._label('release', block=False)
        self._state.q.remove(('request', c, self._id))
        self.send(('release', self.logical_clock(), self._id), self._state.s)

    def _P_handler_280(self, c2, p):
        self._state.q.add(('request', c2, p))
        self.send(('ack', self.logical_clock(), self._id), p)
    _P_handler_280._labels = None
    _P_handler_280._notlabels = None

    def _P_handler_305(self, p):
        for x in {('request', c, p) for (_ConstantPattern322_, c, _BoundPattern325_) in self._state.q if (_ConstantPattern322_ == 'request') if (_BoundPattern325_ == p)}:
            self._state.q.remove(x)
            break
    _P_handler_305._labels = None
    _P_handler_305._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._Node_ReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_Node_ReceivedEvent_0', PatternExpr_470, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])
    _config_object = {'channel': 'fifo', 'clock': 'lamport'}

    def run(self):
        nprocs = (int(sys.argv[1]) if (len(sys.argv) > 1) else 10)
        nrequests = (int(sys.argv[2]) if (len(sys.argv) > 2) else 1)
        if (not (self.nameof(self._id) == 'aaa')):
            ps = self.new(P, num=nprocs)
            for p in ps:
                self._setup(p, ((ps - {p}), nrequests))
            self._start(ps)
            self.output('ps started')
            super()._label('_st_label_462', block=False)
            p = None

            def UniversalOpExpr_463():
                nonlocal p
                for p in ps:
                    if (not PatternExpr_476.match_iter(self._Node_ReceivedEvent_0, _BoundPattern486_=p)):
                        return False
                return True
            _st_label_462 = 0
            self._timer_start()
            while (_st_label_462 == 0):
                _st_label_462 += 1
                if UniversalOpExpr_463():
                    _st_label_462 += 1
                elif self._timer_expired:
                    _st_label_462 += 1
                else:
                    super()._label('_st_label_462', block=True, timeout=5)
                    _st_label_462 -= 1
            self.end(ps)
        else:
            super()._label('_st_label_492', block=False)
            _st_label_492 = 0
            while (_st_label_492 == 0):
                _st_label_492 += 1
                if False:
                    _st_label_492 += 1
                else:
                    super()._label('_st_label_492', block=True)
                    _st_label_492 -= 1



# -*- generated by 1.0.2 -*-
import da
PatternExpr_208 = da.pat.TuplePattern([da.pat.BoundPattern('_BoundPattern209_'), da.pat.FreePattern('rclock'), da.pat.FreePattern('message')])
_config_object = {}

class TagClock():
    pass

class LamportClock(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_LamportClockReceivedEvent_0', PatternExpr_208, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._LamportClock_handler_207])])

    def setup(self, **rest_235):
        super().setup(**rest_235)
        self._state._logical_clock = 0

    def run(self):
        pass

    def send(self, message, to, channel=None):
        if (not (to == self._id)):
            return super().send((TagClock, self._state._logical_clock, message), to, channel)
        else:
            return super().send(message, to, channel)

    def logical_clock(self):
        return self._state._logical_clock

    def _LamportClock_handler_207(self, rclock, message):
        self._state._logical_clock = (max(self._state._logical_clock, rclock) + 1)
        self.send(message, self._id)
    _LamportClock_handler_207._labels = None
    _LamportClock_handler_207._notlabels = None



# -*- generated by 1.0.2 -*-
import da
PatternExpr_210 = da.pat.TuplePattern([da.pat.ConstantPattern('Promise'), da.pat.BoundPattern('_BoundPattern213_'), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern('a')])
PatternExpr_243 = da.pat.TuplePattern([da.pat.ConstantPattern('Promise'), da.pat.BoundPattern('_BoundPattern246_'), da.pat.FreePattern('n'), da.pat.FreePattern('v'), da.pat.FreePattern(None)])
PatternExpr_273 = da.pat.TuplePattern([da.pat.ConstantPattern('Promise'), da.pat.BoundPattern('_BoundPattern276_'), da.pat.BoundPattern('_BoundPattern277_'), da.pat.BoundPattern('_BoundPattern278_'), da.pat.FreePattern('a')])
PatternExpr_327 = da.pat.TuplePattern([da.pat.ConstantPattern('TwoAv'), da.pat.BoundPattern('_BoundPattern330_'), da.pat.BoundPattern('_BoundPattern331_'), da.pat.FreePattern('a')])
PatternExpr_413 = da.pat.TuplePattern([da.pat.ConstantPattern('TwoAv'), da.pat.FreePattern('n'), da.pat.FreePattern('v'), da.pat.FreePattern(None)])
PatternExpr_442 = da.pat.TuplePattern([da.pat.ConstantPattern('TwoAv'), da.pat.BoundPattern('_BoundPattern445_'), da.pat.BoundPattern('_BoundPattern446_'), da.pat.FreePattern('a')])
PatternExpr_470 = da.pat.TuplePattern([da.pat.ConstantPattern('TwoB'), da.pat.BoundPattern('_BoundPattern473_'), da.pat.BoundPattern('_BoundPattern474_')])
PatternExpr_504 = da.pat.TuplePattern([da.pat.ConstantPattern('Done')])
PatternExpr_509 = da.pat.BoundPattern('_BoundPattern510_')
PatternExpr_525 = da.pat.TuplePattern([da.pat.ConstantPattern('Prepare'), da.pat.FreePattern('n'), da.pat.FreePattern('p')])
PatternExpr_544 = da.pat.TuplePattern([da.pat.ConstantPattern('TwoAv'), da.pat.FreePattern('vpn'), da.pat.FreePattern('vv'), da.pat.SelfPattern()])
PatternExpr_595 = da.pat.TuplePattern([da.pat.ConstantPattern('OneC'), da.pat.FreePattern('n'), da.pat.FreePattern('v'), da.pat.FreePattern('p')])
PatternExpr_619 = da.pat.TuplePattern([da.pat.ConstantPattern('TwoAv'), da.pat.BoundPattern('_BoundPattern622_'), da.pat.FreePattern(None), da.pat.SelfPattern()])
PatternExpr_658 = da.pat.TuplePattern([da.pat.ConstantPattern('Promise'), da.pat.FreePattern('n'), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_696 = da.pat.TuplePattern([da.pat.ConstantPattern('Promise'), da.pat.BoundPattern('_BoundPattern699_'), da.pat.FreePattern('vn'), da.pat.FreePattern('vv'), da.pat.FreePattern(None)])
PatternExpr_726 = da.pat.TuplePattern([da.pat.ConstantPattern('Promise'), da.pat.BoundPattern('_BoundPattern729_'), da.pat.BoundPattern('_BoundPattern730_'), da.pat.BoundPattern('_BoundPattern731_'), da.pat.FreePattern('a')])
PatternExpr_477 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('TwoB'), da.pat.BoundPattern('_BoundPattern487_'), da.pat.BoundPattern('_BoundPattern488_')])])
PatternExpr_511 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern517_')]), da.pat.TuplePattern([da.pat.ConstantPattern('Done')])])
_config_object = {}
import sys

class Proposer(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._ProposerReceivedEvent_0 = []
        self._ProposerReceivedEvent_1 = []
        self._ProposerReceivedEvent_2 = []
        self._ProposerReceivedEvent_3 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ProposerReceivedEvent_0', PatternExpr_210, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ProposerReceivedEvent_1', PatternExpr_243, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ProposerReceivedEvent_2', PatternExpr_273, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ProposerReceivedEvent_3', PatternExpr_327, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, acceptors, quorumsize, f, nrounds, timeout, **rest_900):
        super().setup(acceptors=acceptors, quorumsize=quorumsize, f=f, nrounds=nrounds, timeout=timeout, **rest_900)
        self._state.acceptors = acceptors
        self._state.quorumsize = quorumsize
        self._state.f = f
        self._state.nrounds = nrounds
        self._state.timeout = timeout
        self._state.propNum = (0, self._id)
        self._state.propVal = self._id

    def run(self):
        count = 0
        while (count < self._state.nrounds):
            self.work()
            super()._label('prepare', block=False)
            self.send(('Prepare', self._state.propNum, self._id), self._state.acceptors)
            super()._label('_st_label_205', block=False)
            _st_label_205 = 0
            self._timer_start()
            while (_st_label_205 == 0):
                _st_label_205 += 1
                if (len({a for (_, _, (_ConstantPattern228_, _BoundPattern230_, _, _, a)) in self._ProposerReceivedEvent_0 if (_ConstantPattern228_ == 'Promise') if (_BoundPattern230_ == self._state.propNum)}) > self._state.quorumsize):
                    super()._label('propose', block=False)
                    (_, voted) = max(({(n, v) for (_, _, (_ConstantPattern262_, _BoundPattern264_, n, v, _)) in self._ProposerReceivedEvent_1 if (_ConstantPattern262_ == 'Promise') if (_BoundPattern264_ == self._state.propNum) if (len({a for (_, _, (_ConstantPattern291_, _BoundPattern293_, _BoundPattern294_, _BoundPattern295_, a)) in self._ProposerReceivedEvent_2 if (_ConstantPattern291_ == 'Promise') if (_BoundPattern293_ == self._state.propNum) if (_BoundPattern294_ == n) if (_BoundPattern295_ == v)}) > self._state.f)} | {(((- 1), self._id), self._state.propVal)}))
                    self.send(('OneC', self._state.propNum, voted, self._id), self._state.acceptors)
                    super()._label('_st_label_322', block=False)
                    _st_label_322 = 0
                    self._timer_start()
                    while (_st_label_322 == 0):
                        _st_label_322 += 1
                        if (len({a for (_, _, (_ConstantPattern344_, _BoundPattern346_, _BoundPattern347_, a)) in self._ProposerReceivedEvent_3 if (_ConstantPattern344_ == 'TwoAv') if (_BoundPattern346_ == self._state.propNum) if (_BoundPattern347_ == voted)}) > self._state.quorumsize):
                            super()._label('end', block=False)
                            self.output(('Succeeded proposing %s' % (voted,)))
                            count += 1
                            continue
                            _st_label_322 += 1
                        elif self._timer_expired:
                            self.output('Failed to Propose in time, retrying.')
                            _st_label_322 += 1
                        else:
                            super()._label('_st_label_322', block=True, timeout=self._state.timeout)
                            _st_label_322 -= 1
                    else:
                        if (_st_label_322 != 2):
                            continue
                    if (_st_label_322 != 2):
                        break
                    _st_label_205 += 1
                elif self._timer_expired:
                    self.output('Failed to Prepare in time, retrying.')
                    _st_label_205 += 1
                else:
                    super()._label('_st_label_205', block=True, timeout=self._state.timeout)
                    _st_label_205 -= 1
            else:
                if (_st_label_205 != 2):
                    continue
            if (_st_label_205 != 2):
                break
            self._state.propNum = ((self._state.propNum[0] + 1), self._id)
        self.send(('Done',), self._state.acceptors)

class Acceptor(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._AcceptorReceivedEvent_0 = []
        self._AcceptorReceivedEvent_1 = []
        self._AcceptorSentEvent_2 = []
        self._AcceptorReceivedEvent_3 = []
        self._AcceptorSentEvent_5 = []
        self._AcceptorSentEvent_7 = []
        self._AcceptorSentEvent_8 = []
        self._AcceptorReceivedEvent_9 = []
        self._AcceptorReceivedEvent_10 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_0', PatternExpr_413, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_1', PatternExpr_442, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_AcceptorSentEvent_2', PatternExpr_470, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_3', PatternExpr_504, sources=[PatternExpr_509], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_4', PatternExpr_525, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_524]), da.pat.EventPattern(da.pat.SentEvent, '_AcceptorSentEvent_5', PatternExpr_544, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_6', PatternExpr_595, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_594]), da.pat.EventPattern(da.pat.SentEvent, '_AcceptorSentEvent_7', PatternExpr_619, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_AcceptorSentEvent_8', PatternExpr_658, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_9', PatternExpr_696, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_10', PatternExpr_726, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, acceptors, proposers, quorumsize, f, **rest_900):
        super().setup(acceptors=acceptors, proposers=proposers, quorumsize=quorumsize, f=f, **rest_900)
        self._state.acceptors = acceptors
        self._state.proposers = proposers
        self._state.quorumsize = quorumsize
        self._state.f = f
        self._state.peers = (self._state.acceptors | self._state.proposers)

    def run(self):
        while True:
            super()._label('_st_label_410', block=False)
            n = v = a = None

            def ExistentialOpExpr_411():
                nonlocal n, v, a
                for (_, _, (_ConstantPattern431_, n, v, _)) in self._AcceptorReceivedEvent_0:
                    if (_ConstantPattern431_ == 'TwoAv'):
                        if ((len({a for (_, _, (_ConstantPattern459_, _BoundPattern461_, _BoundPattern462_, a)) in self._AcceptorReceivedEvent_1 if (_ConstantPattern459_ == 'TwoAv') if (_BoundPattern461_ == n) if (_BoundPattern462_ == v)}) > self._state.quorumsize) and (not PatternExpr_477.match_iter(self._AcceptorSentEvent_2, _BoundPattern487_=n, _BoundPattern488_=v, SELF_ID=self._id))):
                            return True
                return False
            p = None

            def UniversalOpExpr_496():
                nonlocal p
                for p in self._state.proposers:
                    if (not PatternExpr_511.match_iter(self._AcceptorReceivedEvent_3, _BoundPattern517_=p, SELF_ID=self._id)):
                        return False
                return True
            _st_label_410 = 0
            while (_st_label_410 == 0):
                _st_label_410 += 1
                if ExistentialOpExpr_411():
                    self.send(('TwoB', n, v), self._state.peers)
                    _st_label_410 += 1
                elif UniversalOpExpr_496():
                    break
                    _st_label_410 += 1
                else:
                    super()._label('_st_label_410', block=True)
                    _st_label_410 -= 1
            else:
                if (_st_label_410 != 2):
                    continue
            if (_st_label_410 != 2):
                break

    def maxpromised(self):
        return max(({n for (_, _, (_ConstantPattern676_, n, _, _, _)) in self._AcceptorSentEvent_8 if (_ConstantPattern676_ == 'Promise')} | {((- 2), self._id)}))

    def islegal(self, n, v):
        voted = {(vn, vv) for (_, _, (_ConstantPattern715_, _BoundPattern717_, vn, vv, _)) in self._AcceptorReceivedEvent_9 if (_ConstantPattern715_ == 'Promise') if (_BoundPattern717_ == n) if (len({a for (_, _, (_ConstantPattern744_, _BoundPattern746_, _BoundPattern747_, _BoundPattern748_, a)) in self._AcceptorReceivedEvent_10 if (_ConstantPattern744_ == 'Promise') if (_BoundPattern746_ == n) if (_BoundPattern747_ == vn) if (_BoundPattern748_ == vv)}) > self._state.f)}
        if (voted and (not (max(voted)[1] is None))):
            return (v == max(voted)[1])
        else:
            return True

    def _Acceptor_handler_524(self, n, p):
        if (n > self.maxpromised()):
            (vn, vv) = max(({(vpn, vv) for (_, _, (_ConstantPattern563_, vpn, vv, _ConstantPattern567_)) in self._AcceptorSentEvent_5 if (_ConstantPattern563_ == 'TwoAv') if (_ConstantPattern567_ == self._id)} | {(((- 1), self._id), None)}))
            self.send(('Promise', n, vn, vv, self._id), self._state.peers)
    _Acceptor_handler_524._labels = None
    _Acceptor_handler_524._notlabels = None

    def _Acceptor_handler_594(self, n, v, p):

        def ExistentialOpExpr_617():
            for (_, _, (_ConstantPattern636_, _BoundPattern638_, _, _ConstantPattern640_)) in self._AcceptorSentEvent_7:
                if (_ConstantPattern636_ == 'TwoAv'):
                    if (_BoundPattern638_ == n):
                        if (_ConstantPattern640_ == self._id):
                            if True:
                                return True
            return False
        if ((n >= self.maxpromised()) and self.islegal(n, v) and (not ExistentialOpExpr_617())):
            self.send(('TwoAv', n, v, self._id), self._state.peers)
    _Acceptor_handler_594._labels = None
    _Acceptor_handler_594._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([])

    def run(self):
        nproposers = (int(sys.argv[1]) if (len(sys.argv) > 1) else 5)
        nacceptors = (int(sys.argv[2]) if (len(sys.argv) > 2) else 10)
        nrounds = (int(sys.argv[3]) if (len(sys.argv) > 3) else 1)
        timeout = (int(sys.argv[4]) if (len(sys.argv) > 4) else 1)
        f = int(((nacceptors - 1) / 3))
        quorum = int(((nacceptors / 2) + f))
        acceptors = self.new(Acceptor, num=nacceptors)
        proposers = self.new(Proposer, num=nproposers)
        self._setup(acceptors, (acceptors, proposers, quorum, f))
        self._setup(proposers, (acceptors, quorum, f, nrounds, timeout))
        self._start(acceptors)
        self._start(proposers)



# -*- generated by 1.0.2 -*-
import da
PatternExpr_190 = da.pat.TuplePattern([da.pat.ConstantPattern('Out'), da.pat.SelfPattern(), da.pat.FreePattern('d')])
PatternExpr_229 = da.pat.TuplePattern([da.pat.ConstantPattern('Leader'), da.pat.FreePattern('leader')])
PatternExpr_257 = da.pat.TuplePattern([da.pat.ConstantPattern('In'), da.pat.SelfPattern()])
PatternExpr_264 = da.pat.BoundPattern('_BoundPattern265_')
PatternExpr_280 = da.pat.TuplePattern([da.pat.ConstantPattern('In'), da.pat.SelfPattern()])
PatternExpr_287 = da.pat.BoundPattern('_BoundPattern288_')
PatternExpr_309 = da.pat.TuplePattern([da.pat.ConstantPattern('Out'), da.pat.FreePattern('v'), da.pat.FreePattern('d')])
PatternExpr_318 = da.pat.FreePattern('source')
PatternExpr_353 = da.pat.TuplePattern([da.pat.ConstantPattern('In'), da.pat.FreePattern('v')])
PatternExpr_360 = da.pat.FreePattern('source')
PatternExpr_379 = da.pat.TuplePattern([da.pat.ConstantPattern('Leader'), da.pat.FreePattern('leader')])
PatternExpr_386 = da.pat.FreePattern('source')
PatternExpr_266 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern272_')]), da.pat.TuplePattern([da.pat.ConstantPattern('In'), da.pat.SelfPattern()])])
PatternExpr_289 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern295_')]), da.pat.TuplePattern([da.pat.ConstantPattern('In'), da.pat.SelfPattern()])])
_config_object = {}
import sys
import random

class P(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._PReceivedEvent_0 = []
        self._PReceivedEvent_1 = []
        self._PReceivedEvent_2 = []
        self._PReceivedEvent_3 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_0', PatternExpr_190, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_1', PatternExpr_229, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_2', PatternExpr_257, sources=[PatternExpr_264], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_3', PatternExpr_280, sources=[PatternExpr_287], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_4', PatternExpr_309, sources=[PatternExpr_318], destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_308]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_5', PatternExpr_353, sources=[PatternExpr_360], destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_352]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_6', PatternExpr_379, sources=[PatternExpr_386], destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_378])])

    def setup(self, left, right, **rest_481):
        super().setup(left=left, right=right, **rest_481)
        self._state.left = left
        self._state.right = right
        pass

    def run(self):
        distance = 1
        while True:
            self.send(('Out', self._id, distance), {self._state.left, self._state.right})
            super()._label('_st_label_187', block=False)
            d = None

            def ExistentialOpExpr_188():
                nonlocal d
                for (_, _, (_ConstantPattern207_, _ConstantPattern209_, d)) in self._PReceivedEvent_0:
                    if (_ConstantPattern207_ == 'Out'):
                        if (_ConstantPattern209_ == self._id):
                            if True:
                                return True
                return False
            leader = None

            def ExistentialOpExpr_227():
                nonlocal leader
                for (_, _, (_ConstantPattern244_, leader)) in self._PReceivedEvent_1:
                    if (_ConstantPattern244_ == 'Leader'):
                        if True:
                            return True
                return False
            _st_label_187 = 0
            while (_st_label_187 == 0):
                _st_label_187 += 1
                if ExistentialOpExpr_188():
                    self.output(('I am leader at distance %d!' % d))
                    self.send(('Leader', self._id), {self._state.left, self._state.right})
                    break
                    _st_label_187 += 1
                elif ExistentialOpExpr_227():
                    self.output('Leader is', leader)
                    break
                    _st_label_187 += 1
                elif (PatternExpr_266.match_iter(self._PReceivedEvent_2, _BoundPattern272_=self._state.left, SELF_ID=self._id) and PatternExpr_289.match_iter(self._PReceivedEvent_3, _BoundPattern295_=self._state.right, SELF_ID=self._id)):
                    distance *= 2
                    for attr in dir(self):
                        if (attr.find('ReceivedEvent_') != (- 1)):
                            getattr(self, attr).clear()
                    _st_label_187 += 1
                else:
                    super()._label('_st_label_187', block=True)
                    _st_label_187 -= 1
            else:
                if (_st_label_187 != 2):
                    continue
            if (_st_label_187 != 2):
                break

    def _P_handler_308(self, v, d, source):
        if (v > self._id):
            if (d > 1):
                self.send(('Out', v, (d - 1)), (self._state.right if (source == self._state.left) else self._state.left))
            elif (d == 1):
                self.send(('In', v), source)
    _P_handler_308._labels = None
    _P_handler_308._notlabels = None

    def _P_handler_352(self, v, source):
        if (v > self._id):
            self.send(('In', v), (self._state.right if (source == self._state.left) else self._state.left))
    _P_handler_352._labels = None
    _P_handler_352._notlabels = None

    def _P_handler_378(self, leader, source):
        self.send(('Leader', leader), (self._state.right if (source == self._state.left) else self._state.left))
    _P_handler_378._labels = None
    _P_handler_378._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([])
    _config_object = {'channel': 'fifo'}

    def run(self):
        n = (int(sys.argv[1]) if (len(sys.argv) > 1) else 10)
        topology = list(self.new(P, num=n))
        random.shuffle(topology)
        for (i, p) in enumerate(topology):
            if (i == (len(topology) - 1)):
                self._setup({p}, (topology[(i - 1)], topology[0]))
            else:
                self._setup({p}, (topology[(i - 1)], topology[(i + 1)]))
        self._start(topology)



# Compiler package for Distalgo

from da.compiler.pygen import PythonGenerator
from da.compiler.parser import Parser, daast_from_file, daast_from_str
from da.compiler.ui import dafile_to_pyast, dafile_to_pyfile, main

__all__ = ['PythonGenerator', 'Parser',
           'daast_from_file', 'daast_from_str',
           'dafile_to_pyast', 'dafile_to_pyfile',
           'main']



# -*- generated by 1.0.2 -*-
import da
PatternExpr_191 = da.pat.TuplePattern([da.pat.ConstantPattern('ack'), da.pat.FreePattern('c'), da.pat.BoundPattern('_BoundPattern196_')])
PatternExpr_233 = da.pat.TuplePattern([da.pat.ConstantPattern('request'), da.pat.FreePattern('c'), da.pat.BoundPattern('_BoundPattern237_')])
PatternExpr_263 = da.pat.TuplePattern([da.pat.ConstantPattern('request'), da.pat.FreePattern('c'), da.pat.FreePattern('p')])
PatternExpr_277 = da.pat.TuplePattern([da.pat.ConstantPattern('request'), da.pat.FreePattern('c'), da.pat.SelfPattern()])
PatternExpr_307 = da.pat.TuplePattern([da.pat.ConstantPattern('ack'), da.pat.FreePattern('c2'), da.pat.BoundPattern('_BoundPattern312_')])
PatternExpr_381 = da.pat.TuplePattern([da.pat.ConstantPattern('Done')])
PatternExpr_386 = da.pat.BoundPattern('_BoundPattern388_')
PatternExpr_389 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern395_')]), da.pat.TuplePattern([da.pat.ConstantPattern('Done')])])
PatternExpr_471 = da.pat.TuplePattern([da.pat.ConstantPattern('Done')])
PatternExpr_476 = da.pat.BoundPattern('_BoundPattern477_')
PatternExpr_478 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern484_')]), da.pat.TuplePattern([da.pat.ConstantPattern('Done')])])
_config_object = {}
import sys

class P(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._PReceivedEvent_0 = []
        self._PReceivedEvent_1 = []
        self._PSentEvent_3 = []
        self._PReceivedEvent_4 = []
        self._PReceivedEvent_5 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_0', PatternExpr_191, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_1', PatternExpr_233, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_2', PatternExpr_263, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_262]), da.pat.EventPattern(da.pat.SentEvent, '_PSentEvent_3', PatternExpr_277, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_4', PatternExpr_307, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_5', PatternExpr_381, sources=[PatternExpr_386], destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, s, n, **rest_494):
        super().setup(s=s, n=n, **rest_494)
        self._state.s = s
        self._state.n = n
        pass

    def run(self):

        def anounce():
            self.output('In cs!')
        for i in range(self._state.n):
            self.cs(anounce)
        self.send(('Done',), self.parent())
        super()._label('_st_label_378', block=False)
        _st_label_378 = 0
        while (_st_label_378 == 0):
            _st_label_378 += 1
            if PatternExpr_389.match_iter(self._PReceivedEvent_5, _BoundPattern395_=self.parent(), SELF_ID=self._id):
                _st_label_378 += 1
            else:
                super()._label('_st_label_378', block=True)
                _st_label_378 -= 1

    def cs(self, task):
        super()._label('request', block=False)
        ownc = self.logical_clock()
        self.send(('request', ownc, self._id), self._state.s)
        super()._label('cs', block=False)
        p = c = None

        def UniversalOpExpr_183():
            nonlocal p, c
            for p in self._state.s:

                def ExistentialOpExpr_189(p):
                    nonlocal c
                    for (_, _, (_ConstantPattern207_, c, _BoundPattern210_)) in self._PReceivedEvent_0:
                        if (_ConstantPattern207_ == 'ack'):
                            if (_BoundPattern210_ == p):
                                if (c > ownc):
                                    return True
                    return False
                if (not ExistentialOpExpr_189(p=p)):
                    return False
            return True
        _st_label_182 = 0
        while (_st_label_182 == 0):
            _st_label_182 += 1
            if UniversalOpExpr_183():
                _st_label_182 += 1
            else:
                super()._label('cs', block=True)
                _st_label_182 -= 1
        task()
        super()._label('release', block=False)
        self.output('Release!')

        def ExistentialOpExpr_231(p):
            for (_, _, (_ConstantPattern248_, c, _BoundPattern251_)) in self._PReceivedEvent_1:
                if (_ConstantPattern248_ == 'request'):
                    if (_BoundPattern251_ == p):
                        if ((c, p) >= (ownc, self._id)):
                            return True
            return False
        self.send(('ack', self.logical_clock(), self._id), {p for p in self._state.s if ExistentialOpExpr_231(p=p)})

    def _P_handler_262(self, c, p):

        def UniversalOpExpr_300(c):
            for p in self._state.s:

                def ExistentialOpExpr_305(p, c):
                    for (_, _, (_ConstantPattern323_, c2, _BoundPattern326_)) in self._PReceivedEvent_4:
                        if (_ConstantPattern323_ == 'ack'):
                            if (_BoundPattern326_ == p):
                                if (c2 > c):
                                    return True
                    return False
                if (not ExistentialOpExpr_305(p=p, c=c)):
                    return False
            return True
        ownc = max(({c for (_, _, (_ConstantPattern293_, c, _ConstantPattern296_)) in self._PSentEvent_3 if (_ConstantPattern293_ == 'request') if (_ConstantPattern296_ == self._id) if (not UniversalOpExpr_300(c=c))} | {(- 1)}))
        if ((ownc == (- 1)) or ((c, p) < (ownc, self._id))):
            self.send(('ack', self.logical_clock(), self._id), p)
    _P_handler_262._labels = None
    _P_handler_262._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._Node_ReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_Node_ReceivedEvent_0', PatternExpr_471, sources=[PatternExpr_476], destinations=None, timestamps=None, record_history=True, handlers=[])])
    _config_object = {'clock': 'Lamport'}

    def run(self):
        nprocs = (int(sys.argv[1]) if (len(sys.argv) > 1) else 10)
        nrounds = (int(sys.argv[2]) if (len(sys.argv) > 2) else 1)
        ps = self.new(P, num=nprocs)
        for p in ps:
            self._setup({p}, ((ps - {p}), nrounds))
        self._start(ps)
        super()._label('_st_label_463', block=False)
        p = None

        def UniversalOpExpr_464():
            nonlocal p
            for p in ps:
                if (not PatternExpr_478.match_iter(self._Node_ReceivedEvent_0, _BoundPattern484_=p)):
                    return False
            return True
        _st_label_463 = 0
        while (_st_label_463 == 0):
            _st_label_463 += 1
            if UniversalOpExpr_464():
                _st_label_463 += 1
            else:
                super()._label('_st_label_463', block=True)
                _st_label_463 -= 1
        self.output('All processes done.')
        self.send(('Done',), ps)



# -*- generated by 1.0.2 -*-
import da
PatternExpr_256 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_261 = da.pat.BoundPattern('_BoundPattern262_')
PatternExpr_276 = da.pat.TuplePattern([da.pat.ConstantPattern('Value'), da.pat.FreePattern('v')])
PatternExpr_263 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern269_')]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
_config_object = {'channel': 'Reliable', 'handling': 'one'}
import sys

class P(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._PReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_0', PatternExpr_256, sources=[PatternExpr_261], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_1', PatternExpr_276, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_275])])

    def setup(self, ps, v, maxfail, **rest_361):
        super().setup(ps=ps, v=v, maxfail=maxfail, **rest_361)
        self._state.ps = ps
        self._state.v = v
        self._state.maxfail = maxfail
        self._state.x = (- 1)
        self._state.V = {self._state.v: False}
        self._state.receiveflag = False

    def run(self):
        super()._label('start', block=False)
        for i in range(self._state.maxfail):
            super()._label('one_round', block=False)
            for k in self._state.V:
                if (not self._state.V[k]):
                    self.send(('Value', k), self._state.ps)
                    self._state.V[k] = True
            super()._label('_st_label_221', block=False)
            _st_label_221 = 0
            while (_st_label_221 == 0):
                _st_label_221 += 1
                if self._state.receiveflag:
                    _st_label_221 += 1
                else:
                    super()._label('_st_label_221', block=True)
                    _st_label_221 -= 1
            else:
                if (_st_label_221 != 2):
                    continue
            if (_st_label_221 != 2):
                break
            self._state.receiveflag = False
        super()._label('end', block=False)
        self.send(('done',), self._state.ps)
        self._state.x = max([self._state.v for self._state.v in self._state.V if self._state.V[self._state.v]])
        self.output(('x = %r' % self._state.x))
        super()._label('_st_label_247', block=False)
        p = None

        def UniversalOpExpr_248():
            nonlocal p
            for p in self._state.ps:
                if (not PatternExpr_263.match_iter(self._PReceivedEvent_0, _BoundPattern269_=p, SELF_ID=self._id)):
                    return False
            return True
        _st_label_247 = 0
        while (_st_label_247 == 0):
            _st_label_247 += 1
            if UniversalOpExpr_248():
                _st_label_247 += 1
            else:
                super()._label('_st_label_247', block=True)
                _st_label_247 -= 1

    def _P_handler_275(self, v):
        self._state.receiveflag = True
        if (not (v in self._state.V)):
            self._state.V[v] = False
    _P_handler_275._labels = None
    _P_handler_275._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([])

    def run(self):
        n = (int(sys.argv[1]) if (len(sys.argv) > 1) else 10)
        f = (int(sys.argv[2]) if (len(sys.argv) > 2) else 50)
        ps = self.new(P, num=n)
        for (i, p) in enumerate(list(ps)):
            self._setup({p}, (ps, i, f))
        self._start(ps)



# -*- generated by 1.0.2 -*-
import da
PatternExpr_231 = da.pat.TuplePattern([da.pat.ConstantPattern('request'), da.pat.FreePattern('p')])
PatternExpr_252 = da.pat.TuplePattern([da.pat.ConstantPattern('propose'), da.pat.FreePattern('s'), da.pat.BoundPattern('_BoundPattern257_')])
PatternExpr_275 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.BoundPattern('_BoundPattern278_'), da.pat.FreePattern('p2')])
PatternExpr_312 = da.pat.TuplePattern([da.pat.ConstantPattern('propose'), da.pat.FreePattern('s'), da.pat.FreePattern(None)])
PatternExpr_335 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.FreePattern('s'), da.pat.FreePattern(None)])
PatternExpr_382 = da.pat.TuplePattern([da.pat.ConstantPattern('propose'), da.pat.BoundPattern('_BoundPattern385_'), da.pat.FreePattern(None)])
PatternExpr_405 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.BoundPattern('_BoundPattern408_'), da.pat.FreePattern(None)])
PatternExpr_442 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.BoundPattern('_BoundPattern445_'), da.pat.FreePattern('p')])
PatternExpr_473 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.FreePattern('s'), da.pat.BoundPattern('_BoundPattern477_')])
PatternExpr_537 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_542 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_597 = da.pat.TuplePattern([da.pat.ConstantPattern('1b'), da.pat.BoundPattern('_BoundPattern600_'), da.pat.FreePattern(None)])
PatternExpr_604 = da.pat.FreePattern('a')
PatternExpr_640 = da.pat.TuplePattern([da.pat.ConstantPattern('1b'), da.pat.BoundPattern('_BoundPattern643_'), da.pat.FreePattern('accepted')])
PatternExpr_679 = da.pat.TuplePattern([da.pat.ConstantPattern('propose'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_702 = da.pat.TuplePattern([da.pat.ConstantPattern('2a'), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern706_'), da.pat.FreePattern(None)])
PatternExpr_738 = da.pat.TuplePattern([da.pat.ConstantPattern('2b'), da.pat.BoundPattern('_BoundPattern741_'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_765 = da.pat.TuplePattern([da.pat.ConstantPattern('2b'), da.pat.BoundPattern('_BoundPattern768_'), da.pat.BoundPattern('_BoundPattern769_'), da.pat.BoundPattern('_BoundPattern770_')])
PatternExpr_773 = da.pat.FreePattern('a')
PatternExpr_798 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.BoundPattern('_BoundPattern801_'), da.pat.BoundPattern('_BoundPattern802_')])
PatternExpr_830 = da.pat.TuplePattern([da.pat.ConstantPattern('1b'), da.pat.FreePattern('b'), da.pat.FreePattern(None)])
PatternExpr_856 = da.pat.TuplePattern([da.pat.ConstantPattern('2b'), da.pat.FreePattern('b'), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_888 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_908 = da.pat.TuplePattern([da.pat.ConstantPattern('1b'), da.pat.FreePattern('b'), da.pat.FreePattern(None)])
PatternExpr_933 = da.pat.TuplePattern([da.pat.ConstantPattern('2b'), da.pat.FreePattern('b'), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_964 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_805 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.BoundPattern('_BoundPattern815_'), da.pat.BoundPattern('_BoundPattern816_')])])
PatternExpr_893 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_969 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_1038 = da.pat.TuplePattern([da.pat.ConstantPattern('1a'), da.pat.FreePattern(None)])
PatternExpr_1044 = da.pat.FreePattern('leader')
PatternExpr_1053 = da.pat.TuplePattern([da.pat.ConstantPattern('1a'), da.pat.FreePattern('b')])
PatternExpr_1075 = da.pat.TuplePattern([da.pat.ConstantPattern('2a'), da.pat.FreePattern('b'), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_1120 = da.pat.TuplePattern([da.pat.ConstantPattern('2a'), da.pat.FreePattern('b'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_1161 = da.pat.TuplePattern([da.pat.ConstantPattern('2a'), da.pat.FreePattern('b'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_1172 = da.pat.FreePattern('leader')
PatternExpr_1181 = da.pat.TuplePattern([da.pat.ConstantPattern('1a'), da.pat.FreePattern('b')])
PatternExpr_1202 = da.pat.TuplePattern([da.pat.ConstantPattern('2a'), da.pat.FreePattern('b'), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_1275 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_1280 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_1383 = da.pat.TuplePattern([da.pat.ConstantPattern('response'), da.pat.FreePattern('cid'), da.pat.FreePattern('result')])
PatternExpr_1587 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_1592 = da.pat.BoundPattern('_BoundPattern1593_')
PatternExpr_1594 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern1600_')]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
_config_object = {}
import sys
import time
import random
NOPS = 10

def operation(i):
    return (lambda state: ((state + [i]), ['result', i, 'on', state]))
operations = {i: operation(i) for i in range(NOPS)}

class Replica(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._ReplicaReceivedEvent_0 = []
        self._ReplicaSentEvent_1 = []
        self._ReplicaReceivedEvent_2 = []
        self._ReplicaSentEvent_3 = []
        self._ReplicaReceivedEvent_4 = []
        self._ReplicaSentEvent_5 = []
        self._ReplicaReceivedEvent_6 = []
        self._ReplicaReceivedEvent_7 = []
        self._ReplicaReceivedEvent_8 = []
        self._ReplicaReceivedEvent_9 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_0', PatternExpr_231, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_ReplicaSentEvent_1', PatternExpr_252, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_2', PatternExpr_275, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_ReplicaSentEvent_3', PatternExpr_312, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_4', PatternExpr_335, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_ReplicaSentEvent_5', PatternExpr_382, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_6', PatternExpr_405, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_7', PatternExpr_442, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_8', PatternExpr_473, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_9', PatternExpr_537, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, leaders, initial_state, **rest_1614):
        super().setup(leaders=leaders, initial_state=initial_state, **rest_1614)
        self._state.leaders = leaders
        self._state.initial_state = initial_state
        self._state.state = self._state.initial_state
        self._state.slot_num = 1

    def run(self):
        self.serve()

    def serve(self):
        while True:
            self.debug('### replica')
            super()._label('_st_label_228', block=False)
            p2 = s = p = None

            def ExistentialOpExpr_229():
                nonlocal p2, s, p
                for (_, _, (_ConstantPattern246_, p)) in self._ReplicaReceivedEvent_0:
                    if (_ConstantPattern246_ == 'request'):

                        def UniversalOpExpr_250(p):
                            nonlocal p2, s
                            for (_, _, (_ConstantPattern268_, s, _BoundPattern271_)) in self._ReplicaSentEvent_1:
                                if (_ConstantPattern268_ == 'propose'):
                                    if (_BoundPattern271_ == p):

                                        def ExistentialOpExpr_273(s, p):
                                            nonlocal p2
                                            for (_, _, (_ConstantPattern291_, _BoundPattern293_, p2)) in self._ReplicaReceivedEvent_2:
                                                if (_ConstantPattern291_ == 'decision'):
                                                    if (_BoundPattern293_ == s):
                                                        if (not (p2 == p)):
                                                            return True
                                            return False
                                        if (not ExistentialOpExpr_273(s=s, p=p)):
                                            return False
                            return True
                        if UniversalOpExpr_250(p=p):
                            return True
                return False
            p = None

            def ExistentialOpExpr_440():
                nonlocal p
                for (_, _, (_ConstantPattern457_, _BoundPattern459_, p)) in self._ReplicaReceivedEvent_7:
                    if (_ConstantPattern457_ == 'decision'):
                        if (_BoundPattern459_ == self._state.slot_num):
                            if True:
                                return True
                return False
            _st_label_228 = 0
            while (_st_label_228 == 0):
                _st_label_228 += 1
                if ExistentialOpExpr_229():
                    self.debug('### replica 1', 'p', p)
                    used_s = max((({s for (_, _, (_ConstantPattern327_, s, _)) in self._ReplicaSentEvent_3 if (_ConstantPattern327_ == 'propose')} | {s for (_, _, (_ConstantPattern350_, s, _)) in self._ReplicaReceivedEvent_4 if (_ConstantPattern350_ == 'decision')}) or {0}))
                    self.debug('used_s', used_s)

                    def ExistentialOpExpr_380(s):
                        for (_, _, (_ConstantPattern397_, _BoundPattern399_, _)) in self._ReplicaSentEvent_5:
                            if (_ConstantPattern397_ == 'propose'):
                                if (_BoundPattern399_ == s):
                                    if True:
                                        return True
                        return False

                    def ExistentialOpExpr_403(s):
                        for (_, _, (_ConstantPattern420_, _BoundPattern422_, _)) in self._ReplicaReceivedEvent_6:
                            if (_ConstantPattern420_ == 'decision'):
                                if (_BoundPattern422_ == s):
                                    if True:
                                        return True
                        return False
                    s = min({s for s in range(1, ((used_s + 1) + 1)) if (not (ExistentialOpExpr_380(s=s) or ExistentialOpExpr_403(s=s)))})
                    self.debug('s', s, 'p', p)
                    self.send(('propose', s, p), self._state.leaders)
                    _st_label_228 += 1
                elif ExistentialOpExpr_440():
                    self.debug('### replica 2', 'p', p)
                    s = None

                    def ExistentialOpExpr_471():
                        nonlocal s
                        for (_, _, (_ConstantPattern488_, s, _BoundPattern491_)) in self._ReplicaReceivedEvent_8:
                            if (_ConstantPattern488_ == 'decision'):
                                if (_BoundPattern491_ == p):
                                    if (s < self._state.slot_num):
                                        return True
                        return False
                    if (not ExistentialOpExpr_471()):
                        (client, cmd_id, op) = p
                        (self._state.state, result) = operations[op](self._state.state)
                        self.debug('### replica 2.1', 'state', self._state.state, 'result', result)
                        self.send(('response', cmd_id, result), client)
                    self._state.slot_num += 1
                    self.debug('slot_num', self._state.slot_num)
                    _st_label_228 += 1
                elif PatternExpr_542.match_iter(self._ReplicaReceivedEvent_9, SELF_ID=self._id):
                    self.output('terminating')
                    break
                    _st_label_228 += 1
                else:
                    super()._label('_st_label_228', block=True)
                    _st_label_228 -= 1
            else:
                if (_st_label_228 != 2):
                    continue
            if (_st_label_228 != 2):
                break

class Leader(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._LeaderReceivedEvent_0 = []
        self._LeaderReceivedEvent_1 = []
        self._LeaderReceivedEvent_2 = []
        self._LeaderSentEvent_3 = []
        self._LeaderReceivedEvent_4 = []
        self._LeaderReceivedEvent_5 = []
        self._LeaderSentEvent_6 = []
        self._LeaderReceivedEvent_7 = []
        self._LeaderReceivedEvent_8 = []
        self._LeaderReceivedEvent_9 = []
        self._LeaderReceivedEvent_10 = []
        self._LeaderReceivedEvent_11 = []
        self._LeaderReceivedEvent_12 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_0', PatternExpr_597, sources=[PatternExpr_604], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_1', PatternExpr_640, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_2', PatternExpr_679, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_LeaderSentEvent_3', PatternExpr_702, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_4', PatternExpr_738, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_5', PatternExpr_765, sources=[PatternExpr_773], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.SentEvent, '_LeaderSentEvent_6', PatternExpr_798, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_7', PatternExpr_830, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_8', PatternExpr_856, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_9', PatternExpr_888, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_10', PatternExpr_908, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_11', PatternExpr_933, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_12', PatternExpr_964, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, acceptors, replicas, **rest_1614):
        super().setup(acceptors=acceptors, replicas=replicas, **rest_1614)
        self._state.acceptors = acceptors
        self._state.replicas = replicas
        self._state.ballot = (0, self._id)

    def run(self):
        self.lead()

    def lead(self):
        while True:
            self.debug('### leader')
            self.send(('1a', self._state.ballot), self._state.acceptors)
            super()._label('_st_label_592', block=False)
            b = None

            def ExistentialOpExpr_906():
                nonlocal b
                for (_, _, (_ConstantPattern923_, b, _)) in self._LeaderReceivedEvent_10:
                    if (_ConstantPattern923_ == '1b'):
                        if (b > self._state.ballot):
                            return True
                return False
            b = None

            def ExistentialOpExpr_931():
                nonlocal b
                for (_, _, (_ConstantPattern949_, b, _, _)) in self._LeaderReceivedEvent_11:
                    if (_ConstantPattern949_ == '2b'):
                        if (b > self._state.ballot):
                            return True
                return False
            _st_label_592 = 0
            while (_st_label_592 == 0):
                _st_label_592 += 1
                if (len({a for (_, (_, _, a), (_ConstantPattern615_, _BoundPattern617_, _)) in self._LeaderReceivedEvent_0 if (_ConstantPattern615_ == '1b') if (_BoundPattern617_ == self._state.ballot)}) > (len(self._state.acceptors) / 2)):
                    self.debug('### leader 1')
                    for (s, p) in self.pmax({t for (_, _, (_ConstantPattern656_, _BoundPattern658_, accepted)) in self._LeaderReceivedEvent_1 if (_ConstantPattern656_ == '1b') if (_BoundPattern658_ == self._state.ballot) for t in accepted}):
                        self.send(('2a', self._state.ballot, s, p), self._state.acceptors)
                    while True:
                        super()._label('_st_label_676', block=False)
                        p = s = None

                        def ExistentialOpExpr_677():
                            nonlocal p, s
                            for (_, _, (_ConstantPattern694_, s, p)) in self._LeaderReceivedEvent_2:
                                if (_ConstantPattern694_ == 'propose'):

                                    def ExistentialOpExpr_700(s):
                                        for (_, _, (_ConstantPattern718_, _, _BoundPattern721_, _)) in self._LeaderSentEvent_3:
                                            if (_ConstantPattern718_ == '2a'):
                                                if (_BoundPattern721_ == s):
                                                    if True:
                                                        return True
                                        return False
                                    if (not ExistentialOpExpr_700(s=s)):
                                        return True
                            return False
                        p = s = a = None

                        def ExistentialOpExpr_736():
                            nonlocal p, s, a
                            for (_, _, (_ConstantPattern754_, _BoundPattern756_, s, p)) in self._LeaderReceivedEvent_4:
                                if (_ConstantPattern754_ == '2b'):
                                    if (_BoundPattern756_ == self._state.ballot):
                                        if ((len({a for (_, (_, _, a), (_ConstantPattern784_, _BoundPattern786_, _BoundPattern787_, _BoundPattern788_)) in self._LeaderReceivedEvent_5 if (_ConstantPattern784_ == '2b') if (_BoundPattern786_ == self._state.ballot) if (_BoundPattern787_ == s) if (_BoundPattern788_ == p)}) > (len(self._state.acceptors) / 2)) and (not PatternExpr_805.match_iter(self._LeaderSentEvent_6, _BoundPattern815_=s, _BoundPattern816_=p, SELF_ID=self._id))):
                                            return True
                            return False
                        b = None

                        def ExistentialOpExpr_828():
                            nonlocal b
                            for (_, _, (_ConstantPattern846_, b, _)) in self._LeaderReceivedEvent_7:
                                if (_ConstantPattern846_ == '1b'):
                                    if (b > self._state.ballot):
                                        return True
                            return False
                        b = None

                        def ExistentialOpExpr_854():
                            nonlocal b
                            for (_, _, (_ConstantPattern872_, b, _, _)) in self._LeaderReceivedEvent_8:
                                if (_ConstantPattern872_ == '2b'):
                                    if (b > self._state.ballot):
                                        return True
                            return False
                        _st_label_676 = 0
                        while (_st_label_676 == 0):
                            _st_label_676 += 1
                            if ExistentialOpExpr_677():
                                self.debug('### leader 1.1')
                                self.send(('2a', self._state.ballot, s, p), self._state.acceptors)
                                _st_label_676 += 1
                            elif ExistentialOpExpr_736():
                                self.debug('### leader 1.2')
                                self.send(('decision', s, p), self._state.replicas)
                                _st_label_676 += 1
                            elif (ExistentialOpExpr_828() or ExistentialOpExpr_854()):
                                self.debug('### leader 1.3')
                                break
                                _st_label_676 += 1
                            elif PatternExpr_893.match_iter(self._LeaderReceivedEvent_9, SELF_ID=self._id):
                                break
                                _st_label_676 += 1
                            else:
                                super()._label('_st_label_676', block=True)
                                _st_label_676 -= 1
                        else:
                            if (_st_label_676 != 2):
                                continue
                        if (_st_label_676 != 2):
                            break
                    _st_label_592 += 1
                elif (ExistentialOpExpr_906() or ExistentialOpExpr_931()):
                    self.output('### leader 2')
                    pass
                    _st_label_592 += 1
                elif PatternExpr_969.match_iter(self._LeaderReceivedEvent_12, SELF_ID=self._id):
                    self.output('terminating')
                    break
                    _st_label_592 += 1
                else:
                    super()._label('_st_label_592', block=True)
                    _st_label_592 -= 1
            else:
                if (_st_label_592 != 2):
                    continue
            if (_st_label_592 != 2):
                break
            self._state.ballot = ((self._state.ballot[0] + 1), self._id)

    def pmax(self, pvals):

        def UniversalOpExpr_1007(s, b):
            for (b2, _BoundPattern1012_, _) in pvals:
                if (_BoundPattern1012_ == s):
                    if (not (b2 <= b)):
                        return False
            return True
        return {(s, p) for (b, s, p) in pvals if UniversalOpExpr_1007(s=s, b=b)}

class Acceptor(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._AcceptorReceivedEvent_1 = []
        self._AcceptorReceivedEvent_2 = []
        self._AcceptorReceivedEvent_3 = []
        self._AcceptorReceivedEvent_5 = []
        self._AcceptorReceivedEvent_6 = []
        self._AcceptorReceivedEvent_7 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_0', PatternExpr_1038, sources=[PatternExpr_1044], destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_1037]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_1', PatternExpr_1053, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_2', PatternExpr_1075, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_3', PatternExpr_1120, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_4', PatternExpr_1161, sources=[PatternExpr_1172], destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_1160]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_5', PatternExpr_1181, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_6', PatternExpr_1202, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_7', PatternExpr_1275, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, **rest_1614):
        super().setup(**rest_1614)
        pass

    def run(self):
        super()._label('_st_label_1272', block=False)
        _st_label_1272 = 0
        while (_st_label_1272 == 0):
            _st_label_1272 += 1
            if PatternExpr_1280.match_iter(self._AcceptorReceivedEvent_7, SELF_ID=self._id):
                _st_label_1272 += 1
            else:
                super()._label('_st_label_1272', block=True)
                _st_label_1272 -= 1
        self.output('terminating')

    def pmax(self, pvals):

        def UniversalOpExpr_1255(s, b):
            for (b2, _BoundPattern1260_, _) in pvals:
                if (_BoundPattern1260_ == s):
                    if (not (b2 <= b)):
                        return False
            return True
        return {(s, p) for (b, s, p) in pvals if UniversalOpExpr_1255(s=s, b=b)}

    def _Acceptor_handler_1037(self, leader):
        ballot = max((({b for (_, _, (_ConstantPattern1068_, b)) in self._AcceptorReceivedEvent_1 if (_ConstantPattern1068_ == '1a')} | {b for (_, _, (_ConstantPattern1092_, b, _, _)) in self._AcceptorReceivedEvent_2 if (_ConstantPattern1092_ == '2a')}) or {((- 1), (- 1))}))
        accepted = {(ballot, s, p) for (s, p) in self.pmax({(b, s, p) for (_, _, (_ConstantPattern1137_, b, s, p)) in self._AcceptorReceivedEvent_3 if (_ConstantPattern1137_ == '2a')})}
        self.send(('1b', ballot, accepted), leader)
    _Acceptor_handler_1037._labels = None
    _Acceptor_handler_1037._notlabels = None

    def _Acceptor_handler_1160(self, b, s, p, leader):
        ballot = max((({b for (_, _, (_ConstantPattern1195_, b)) in self._AcceptorReceivedEvent_5 if (_ConstantPattern1195_ == '1a')} | {b for (_, _, (_ConstantPattern1218_, b, _, _)) in self._AcceptorReceivedEvent_6 if (_ConstantPattern1218_ == '2a')}) or {((- 1), (- 1))}))
        self.send(('2b', ballot, s, p), leader)
    _Acceptor_handler_1160._labels = None
    _Acceptor_handler_1160._notlabels = None

class Client(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ClientReceivedEvent_0', PatternExpr_1383, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Client_handler_1382])])

    def setup(self, replicas, nops, **rest_1614):
        super().setup(replicas=replicas, nops=nops, **rest_1614)
        self._state.replicas = replicas
        self._state.nops = nops
        self._state.cid = 0
        self._state.results = dict()
        self._state.count = dict()

    def run(self):
        for i in range(self._state.nops):
            self.send(('request', (self._id, self._state.cid, random.randint(0, (NOPS - 1)))), self._state.replicas)
            super()._label('_st_label_1345', block=False)
            _st_label_1345 = 0
            while (_st_label_1345 == 0):
                _st_label_1345 += 1
                if (self._state.cid in self._state.results):
                    _st_label_1345 += 1
                else:
                    super()._label('_st_label_1345', block=True)
                    _st_label_1345 -= 1
            else:
                if (_st_label_1345 != 2):
                    continue
            if (_st_label_1345 != 2):
                break
            self.output('received result', self._state.cid, self._state.results[self._state.cid])
            self._state.cid += 1
        super()._label('_st_label_1359', block=False)
        cid = None

        def UniversalOpExpr_1360():
            nonlocal cid
            for self._state.cid in range(self._state.nops):
                if (not (self._state.count[self._state.cid] == len(self._state.replicas))):
                    return False
            return True
        _st_label_1359 = 0
        while (_st_label_1359 == 0):
            _st_label_1359 += 1
            if UniversalOpExpr_1360():
                _st_label_1359 += 1
            else:
                super()._label('_st_label_1359', block=True)
                _st_label_1359 -= 1
        self.output('terminating')
        self.send(('done',), self.nodeof(self._id))

    def _Client_handler_1382(self, cid, result):
        self.debug('### response', cid, result)
        if (not (cid in self._state.results)):
            self._state.results[cid] = result
        elif (not (self._state.results[cid] == result)):
            self.output('different result', cid, result, 'than', self._state.results[cid])
            raise RuntimeError('Desynced!!!!')
        self._state.count[cid] = (1 if (not (cid in self._state.count)) else (self._state.count[cid] + 1))
    _Client_handler_1382._labels = None
    _Client_handler_1382._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._Node_ReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_Node_ReceivedEvent_0', PatternExpr_1587, sources=[PatternExpr_1592], destinations=None, timestamps=None, record_history=True, handlers=[])])

    def run(self):
        nacceptors = (int(sys.argv[1]) if (len(sys.argv) > 1) else 8)
        nreplicas = (int(sys.argv[2]) if (len(sys.argv) > 2) else 8)
        nleaders = (int(sys.argv[3]) if (len(sys.argv) > 3) else 2)
        nclients = (int(sys.argv[4]) if (len(sys.argv) > 4) else 5)
        nops = (int(sys.argv[5]) if (len(sys.argv) > 5) else 3)
        acceptors = self.new(Acceptor, (), num=nacceptors)
        replicas = self.new(Replica, num=nreplicas)
        leaders = self.new(Leader, (acceptors, replicas), num=nleaders)
        initial_state = []
        self._setup(replicas, (leaders, initial_state))
        clients = self.new(Client, (replicas, nops), num=nclients)
        self._start(acceptors)
        self._start((replicas | leaders))
        self._start(clients)
        super()._label('_st_label_1578', block=False)
        c = None

        def UniversalOpExpr_1579():
            nonlocal c
            for c in clients:
                if (not PatternExpr_1594.match_iter(self._Node_ReceivedEvent_0, _BoundPattern1600_=c)):
                    return False
            return True
        _st_label_1578 = 0
        while (_st_label_1578 == 0):
            _st_label_1578 += 1
            if UniversalOpExpr_1579():
                _st_label_1578 += 1
            else:
                super()._label('_st_label_1578', block=True)
                _st_label_1578 -= 1
        self.output('All clients done!')
        self.send(('done',), ((acceptors | replicas) | leaders))



# -*- generated by 1.0.2 -*-
import da
PatternExpr_181 = da.pat.TuplePattern([da.pat.ConstantPattern('Election'), da.pat.FreePattern('p')])
PatternExpr_205 = da.pat.TuplePattern([da.pat.ConstantPattern('Election'), da.pat.SelfPattern()])
PatternExpr_240 = da.pat.TuplePattern([da.pat.ConstantPattern('Leader'), da.pat.FreePattern('leader')])
PatternExpr_268 = da.pat.TuplePattern([da.pat.ConstantPattern('Leader'), da.pat.FreePattern(None)])
PatternExpr_212 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('Election'), da.pat.SelfPattern()])])
_config_object = {}
import sys

class P(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._PSentEvent_1 = []
        self._PReceivedEvent_3 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_0', PatternExpr_181, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_180]), da.pat.EventPattern(da.pat.SentEvent, '_PSentEvent_1', PatternExpr_205, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_2', PatternExpr_240, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._P_handler_239]), da.pat.EventPattern(da.pat.ReceivedEvent, '_PReceivedEvent_3', PatternExpr_268, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, left, **rest_351):
        super().setup(left=left, **rest_351)
        self._state.left = left
        self._state.leaderid = None

    def run(self):
        self.initiate()
        super()._label('_st_label_265', block=False)

        def ExistentialOpExpr_266():
            for (_, _, (_ConstantPattern282_, _)) in self._PReceivedEvent_3:
                if (_ConstantPattern282_ == 'Leader'):
                    if True:
                        return True
            return False
        _st_label_265 = 0
        while (_st_label_265 == 0):
            _st_label_265 += 1
            if ExistentialOpExpr_266():
                _st_label_265 += 1
            else:
                super()._label('_st_label_265', block=True)
                _st_label_265 -= 1
        self.output('Leader is', self._state.leaderid)

    def initiate(self):
        self.send(('Election', self._id), self._state.left)

    def _P_handler_180(self, p):
        if (p > self._id):
            self.send(('Election', p), self._state.left)
        if (p < self._id):
            if (not PatternExpr_212.match_iter(self._PSentEvent_1, SELF_ID=self._id)):
                self.send(('Election', self._id), self._state.left)
        if (p == self._id):
            self.send(('Leader', self._id), self._state.left)
    _P_handler_180._labels = None
    _P_handler_180._notlabels = None

    def _P_handler_239(self, leader):
        self._state.leaderid = leader
        if (not (leader == self._id)):
            self.send(('Leader', leader), self._state.left)
    _P_handler_239._labels = None
    _P_handler_239._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([])
    _config_object = {'channel': 'fifo'}

    def run(self):
        n = (int(sys.argv[1]) if (len(sys.argv) > 1) else 10)
        ps = list(self.new(P, num=n))
        for (i, p) in enumerate(ps):
            self._setup({p}, (ps[((i + 1) if (i < (len(ps) - 1)) else 0)],))
        self._start(ps)



# -*- generated by 1.0.2 -*-
import da
PatternExpr_377 = da.pat.TuplePattern([da.pat.ConstantPattern('AppendEntries'), da.pat.FreePattern('term'), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_444 = da.pat.TuplePattern([da.pat.ConstantPattern('RequestVoteReply'), da.pat.BoundPattern('_BoundPattern447_'), da.pat.ConstantPattern(True)])
PatternExpr_452 = da.pat.FreePattern('p')
PatternExpr_507 = da.pat.TuplePattern([da.pat.ConstantPattern('AppendEntries'), da.pat.FreePattern('term'), da.pat.FreePattern('leader'), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_644 = da.pat.TuplePattern([da.pat.ConstantPattern('RequestVote'), da.pat.FreePattern('term'), da.pat.FreePattern('candidateId'), da.pat.FreePattern('lastLogIndex'), da.pat.FreePattern('lastLogTerm')])
PatternExpr_701 = da.pat.TuplePattern([da.pat.ConstantPattern('RequestVoteReply'), da.pat.FreePattern('term'), da.pat.ConstantPattern(False)])
PatternExpr_716 = da.pat.TuplePattern([da.pat.ConstantPattern('AppendEntries'), da.pat.FreePattern('term'), da.pat.FreePattern('leaderId'), da.pat.FreePattern('prevLogIndex'), da.pat.FreePattern('prevLogTerm'), da.pat.FreePattern('entries'), da.pat.FreePattern('leaderCommit')])
PatternExpr_833 = da.pat.TuplePattern([da.pat.ConstantPattern('AppendEntriesReply'), da.pat.FreePattern('term'), da.pat.FreePattern('success'), da.pat.FreePattern('updatedIndex')])
PatternExpr_844 = da.pat.FreePattern('server')
PatternExpr_876 = da.pat.TuplePattern([da.pat.ConstantPattern('ClientRequest'), da.pat.FreePattern('serial')])
PatternExpr_883 = da.pat.FreePattern('client')
PatternExpr_1061 = da.pat.TuplePattern([da.pat.ConstantPattern('NotLeader'), da.pat.BoundPattern('_BoundPattern1064_'), da.pat.FreePattern('leader')])
PatternExpr_1098 = da.pat.TuplePattern([da.pat.ConstantPattern('Reply'), da.pat.BoundPattern('_BoundPattern1101_'), da.pat.FreePattern(None)])
PatternExpr_1273 = da.pat.TuplePattern([da.pat.ConstantPattern('Done')])
PatternExpr_1278 = da.pat.BoundPattern('_BoundPattern1279_')
PatternExpr_1280 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern1286_')]), da.pat.TuplePattern([da.pat.ConstantPattern('Done')])])
_config_object = {}
import sys
import random

class Role():
    pass

class Follower(Role):
    pass

class Candidate(Role):
    pass

class Leader(Role):
    pass

class LogEntry():
    'Fictional log entries.'

    def __init__(self, term, client, command):
        self.term = term
        self.client = client
        self.command = command

    def __str__(self):
        sl = ['LogEntry', str(self.term), str(self.client), str(self.command)]
        return ':'.join(sl)

class Server(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._ServerReceivedEvent_0 = []
        self._ServerReceivedEvent_1 = []
        self._ServerReceivedEvent_2 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_0', PatternExpr_377, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_1', PatternExpr_444, sources=[PatternExpr_452], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_2', PatternExpr_507, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_3', PatternExpr_644, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Server_handler_643]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_4', PatternExpr_701, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Server_handler_700]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_5', PatternExpr_716, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Server_handler_715]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_6', PatternExpr_833, sources=[PatternExpr_844], destinations=None, timestamps=None, record_history=None, handlers=[self._Server_handler_832]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ServerReceivedEvent_7', PatternExpr_876, sources=[PatternExpr_883], destinations=None, timestamps=None, record_history=None, handlers=[self._Server_handler_875])])

    def setup(self, peers, maxTimeout, **rest_1295):
        super().setup(peers=peers, maxTimeout=maxTimeout, **rest_1295)
        self._state.peers = peers
        self._state.maxTimeout = maxTimeout
        self._state.currentRole = Follower
        self._state.currentTerm = 0
        self._state.votedFor = None
        self._state.log = [LogEntry(term=0, client=None, command=None)]
        self._state.commitIndex = 0
        self._state.lastApplied = 0
        self._state.nextIndex = dict(((p, 1) for p in self._state.peers))
        self._state.matchIndex = dict(((p, 0) for p in self._state.peers))
        self._state.last_seen_leader = None
        self._state._dispatch_table = {Follower: self.follower_term, Candidate: self.candidate_term, Leader: self.leader_term}

    def run(self):
        while True:
            if (self._state.commitIndex > self._state.lastApplied):
                self._state.lastApplied += 1
                self.commit_to_state_machine()
            termTimeout = (random.randint(int((self._state.maxTimeout / 2)), self._state.maxTimeout) / 1000)
            self._state._dispatch_table[self._state.currentRole](termTimeout)

    def follower_term(self, termTimeout):
        super()._label('_st_label_374', block=False)
        term = None

        def ExistentialOpExpr_375():
            nonlocal term
            for (_, _, (_ConstantPattern397_, term, _, _, _, _, _)) in self._ServerReceivedEvent_0:
                if (_ConstantPattern397_ == 'AppendEntries'):
                    if (term >= self._state.currentTerm):
                        return True
            return False
        _st_label_374 = 0
        self._timer_start()
        while (_st_label_374 == 0):
            _st_label_374 += 1
            if ExistentialOpExpr_375():
                for attr in dir(self):
                    if (attr.find('ReceivedEvent_') != (- 1)):
                        getattr(self, attr).clear()
                _st_label_374 += 1
            elif self._timer_expired:
                self.output('Heartbeat timeout, transitioning to Candidate state.')
                self._state.currentRole = Candidate
                _st_label_374 += 1
            else:
                super()._label('_st_label_374', block=True, timeout=termTimeout)
                _st_label_374 -= 1

    def candidate_term(self, termTimeout):
        super()._label('start_election', block=False)
        self._state.currentTerm += 1
        self.RequestVoteRPC(target=self._state.peers, term=self._state.currentTerm, candidateId=self._id, lastLogIndex=(len(self._state.log) - 1), lastLogTerm=self._state.log[(- 1)].term)
        super()._label('_st_label_439', block=False)
        leader = term = None

        def ExistentialOpExpr_505():
            nonlocal leader, term
            for (_, _, (_ConstantPattern528_, term, leader, _, _, _, _)) in self._ServerReceivedEvent_2:
                if (_ConstantPattern528_ == 'AppendEntries'):
                    if (term >= self._state.currentTerm):
                        return True
            return False
        _st_label_439 = 0
        self._timer_start()
        while (_st_label_439 == 0):
            _st_label_439 += 1
            if (len({p for (_, (_, _, p), (_ConstantPattern463_, _BoundPattern465_, _ConstantPattern466_)) in self._ServerReceivedEvent_1 if (_ConstantPattern463_ == 'RequestVoteReply') if (_BoundPattern465_ == self._state.currentTerm) if (_ConstantPattern466_ == True)}) > (len(self._state.peers) / 2)):
                self.output('Transitioning to Leader.')
                self._state.currentRole = Leader
                self._state.nextIndex = dict(((p, len(self._state.log)) for p in self._state.peers))
                self._state.matchIndex = dict(((p, 0) for p in self._state.peers))
                _st_label_439 += 1
            elif ExistentialOpExpr_505():
                self.output('Elected leader:', leader, 'Reverting to Follower.')
                self._state.currentTerm = term
                self._state.currentRole = Follower
                _st_label_439 += 1
            elif self._timer_expired:
                self.output('Election term', self._state.currentTerm, 'timeout, restarting.')
                _st_label_439 += 1
            else:
                super()._label('_st_label_439', block=True, timeout=termTimeout)
                _st_label_439 -= 1

    def leader_term(self, termTimeout):
        for (server, index) in self._state.nextIndex.items():
            self.AppendEntriesRPC(target=server, term=self._state.currentTerm, leaderId=self._id, prevLogIndex=(index - 1), prevLogTerm=self._state.log[(index - 1)].term, entries=self._state.log[index:], leaderCommit=self._state.commitIndex)
        super()._label('_st_label_589', block=False)
        _st_label_589 = 0
        self._timer_start()
        while (_st_label_589 == 0):
            _st_label_589 += 1
            if (not (self._state.currentRole is Leader)):
                return
                _st_label_589 += 1
            elif self._timer_expired:
                n = i = None

                def ExistentialOpExpr_600():
                    nonlocal n, i
                    for n in range((len(self._state.log) - 1)):
                        if ((n > self._state.commitIndex) and (len({i for i in self._state.matchIndex if (self._state.matchIndex[i] >= n)}) > (len(self._state.peers) / 2)) and (self._state.log[n].term == self._state.currentTerm)):
                            return True
                    return False
                if ExistentialOpExpr_600():
                    self._state.commitIndex = n
                _st_label_589 += 1
            else:
                super()._label('_st_label_589', block=True, timeout=(termTimeout / 2))
                _st_label_589 -= 1

    def update_term(self, term):
        if (self._state.currentTerm < term):
            self._state.currentTerm = term
            self._state.votedFor = None
            self._state.currentRole = Follower

    def is_up_to_date(self, lastLogIndex, lastLogTerm):
        return ((lastLogTerm, lastLogIndex) >= (self._state.log[(- 1)].term, (len(self._state.log) - 1)))

    def commit_to_state_machine(self):
        entry = self._state.log[self._state.lastApplied]
        self.output(entry, ' at index', self._state.lastApplied, 'applied to state machine.')
        if (self._state.currentRole is Leader):
            self.send(('Reply', entry.command, self._id), entry.client)

    def AppendEntriesRPC(self, target, term, leaderId, prevLogIndex, prevLogTerm, entries, leaderCommit):
        self.send(('AppendEntries', term, leaderId, prevLogIndex, prevLogTerm, entries, leaderCommit), target)

    def AppendEntriesReply(self, target, term, success, updatedIndex=None):
        self.send(('AppendEntriesReply', term, success, updatedIndex), target)

    def RequestVoteRPC(self, target, term, candidateId, lastLogIndex, lastLogTerm):
        self.send(('RequestVote', term, candidateId, lastLogIndex, lastLogTerm), target)

    def RequestVoteReply(self, target, term, voteGranted):
        self.send(('RequestVoteReply', term, voteGranted), target)

    def _Server_handler_643(self, term, candidateId, lastLogIndex, lastLogTerm):
        self.update_term(term)
        if (term < self._state.currentTerm):
            self.RequestVoteReply(target=candidateId, term=self._state.currentTerm, voteGranted=False)
        elif (((self._state.votedFor is None) or (self._state.votedFor == candidateId)) and self.is_up_to_date(lastLogIndex, lastLogTerm)):
            self._state.votedFor = candidateId
            self.RequestVoteReply(target=candidateId, term=self._state.currentTerm, voteGranted=True)
        else:
            self.RequestVoteReply(target=candidateId, term=self._state.currentTerm, voteGranted=False)
    _Server_handler_643._labels = None
    _Server_handler_643._notlabels = None

    def _Server_handler_700(self, term):
        self.update_term(term)
    _Server_handler_700._labels = None
    _Server_handler_700._notlabels = None

    def _Server_handler_715(self, term, leaderId, prevLogIndex, prevLogTerm, entries, leaderCommit):
        self.update_term(term)
        if (term < self._state.currentTerm):
            self.AppendEntriesReply(target=leaderId, term=self._state.currentTerm, success=False)
        elif (not ((len(self._state.log) > prevLogIndex) and (self._state.log[prevLogIndex].term == prevLogTerm))):
            self.AppendEntriesReply(target=leaderId, term=self._state.currentTerm, success=False)
        else:
            self._state.last_seen_leader = leaderId
            for (idx, entry) in enumerate(entries):
                idx += (prevLogIndex + 1)
                if (len(self._state.log) <= idx):
                    self._state.log.append(entry)
                elif (not (self._state.log[idx].term == entry.term)):
                    del self._state.log[idx:]
            last_new_index = (prevLogIndex + len(entries))
            if (leaderCommit > self._state.commitIndex):
                self._state.commitIndex = min(leaderCommit, last_new_index)
            self.AppendEntriesReply(target=leaderId, term=self._state.currentTerm, success=True, updatedIndex=last_new_index)
    _Server_handler_715._labels = None
    _Server_handler_715._notlabels = None

    def _Server_handler_832(self, term, success, updatedIndex, server):
        self.update_term(term)
        if (self._state.currentRole is Leader):
            if success:
                self._state.nextIndex[server] = (updatedIndex + 1)
                self._state.matchIndex[server] = updatedIndex
            else:
                self._state.nextIndex[server] -= 1
    _Server_handler_832._labels = None
    _Server_handler_832._notlabels = None

    def _Server_handler_875(self, serial, client):
        if (not (self._state.currentRole is Leader)):
            self.send(('NotLeader', serial, self._state.last_seen_leader), client)
        else:
            self._state.log.append(LogEntry(self._state.currentTerm, client, serial))
    _Server_handler_875._labels = None
    _Server_handler_875._notlabels = None

class Client(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._ClientReceivedEvent_0 = []
        self._ClientReceivedEvent_1 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ClientReceivedEvent_0', PatternExpr_1061, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ClientReceivedEvent_1', PatternExpr_1098, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, servers, nrequests, timeout, **rest_1295):
        super().setup(servers=servers, nrequests=nrequests, timeout=timeout, **rest_1295)
        self._state.servers = servers
        self._state.nrequests = nrequests
        self._state.timeout = timeout
        pass

    def run(self):
        target = random.choice(self._state.servers)
        req = 0
        while (req < self._state.nrequests):
            self.send(('ClientRequest', req), target)
            super()._label('_st_label_1058', block=False)
            leader = None

            def ExistentialOpExpr_1059():
                nonlocal leader
                for (_, _, (_ConstantPattern1077_, _BoundPattern1079_, leader)) in self._ClientReceivedEvent_0:
                    if (_ConstantPattern1077_ == 'NotLeader'):
                        if (_BoundPattern1079_ == req):
                            if (not (leader is None)):
                                return True
                return False

            def ExistentialOpExpr_1096():
                for (_, _, (_ConstantPattern1113_, _BoundPattern1115_, _)) in self._ClientReceivedEvent_1:
                    if (_ConstantPattern1113_ == 'Reply'):
                        if (_BoundPattern1115_ == req):
                            if True:
                                return True
                return False
            _st_label_1058 = 0
            self._timer_start()
            while (_st_label_1058 == 0):
                _st_label_1058 += 1
                if ExistentialOpExpr_1059():
                    self.debug('Wrong server, changing to', leader)
                    target = leader
                    for attr in dir(self):
                        if (attr.find('ReceivedEvent_') != (- 1)):
                            getattr(self, attr).clear()
                    _st_label_1058 += 1
                elif ExistentialOpExpr_1096():
                    self.output('Request', (req + 1), 'complete.')
                    req += 1
                    _st_label_1058 += 1
                elif self._timer_expired:
                    self.debug('Timeout, new random.')
                    target = random.choice(self._state.servers)
                    _st_label_1058 += 1
                else:
                    super()._label('_st_label_1058', block=True, timeout=(self._state.timeout / 1000))
                    _st_label_1058 -= 1
            else:
                if (_st_label_1058 != 2):
                    continue
            if (_st_label_1058 != 2):
                break
        self.send(('Done',), self.parent())

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._Node_ReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_Node_ReceivedEvent_0', PatternExpr_1273, sources=[PatternExpr_1278], destinations=None, timestamps=None, record_history=True, handlers=[])])

    def run(self):
        nservers = (int(sys.argv[1]) if (len(sys.argv) > 1) else 5)
        nclients = (int(sys.argv[2]) if (len(sys.argv) > 2) else 3)
        nrequests = (int(sys.argv[3]) if (len(sys.argv) > 3) else 3)
        maxtimeout = (int(sys.argv[4]) if (len(sys.argv) > 4) else 3000)
        send_failrate = (float(sys.argv[5]) if (len(sys.argv) > 5) else 0.0)
        servers = self.new(Server, num=nservers, send=send_failrate)
        self._setup(servers, (servers, maxtimeout))
        clients = self.new(Client, num=nclients)
        self._setup(clients, (list(servers), nrequests, maxtimeout))
        self._start(servers)
        self._start(clients)
        super()._label('_st_label_1264', block=False)
        c = None

        def UniversalOpExpr_1265():
            nonlocal c
            for c in clients:
                if (not PatternExpr_1280.match_iter(self._Node_ReceivedEvent_0, _BoundPattern1286_=c)):
                    return False
            return True
        _st_label_1264 = 0
        while (_st_label_1264 == 0):
            _st_label_1264 += 1
            if UniversalOpExpr_1265():
                _st_label_1264 += 1
            else:
                super()._label('_st_label_1264', block=True)
                _st_label_1264 -= 1
        self.output('All clients done.')
        self.end(servers)



# -*- generated by 1.0.2 -*-
import da
PatternExpr_382 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_402 = da.pat.TuplePattern([da.pat.ConstantPattern('request'), da.pat.FreePattern('p')])
PatternExpr_419 = da.pat.TuplePattern([da.pat.ConstantPattern('decision'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_387 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_499 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_519 = da.pat.FreePattern('m')
PatternExpr_537 = da.pat.TuplePattern([da.pat.ConstantPattern('p1a'), da.pat.FreePattern(None), da.pat.FreePattern('b')])
PatternExpr_561 = da.pat.TuplePattern([da.pat.ConstantPattern('p2a'), da.pat.FreePattern(None), da.pat.FreePattern('b'), da.pat.FreePattern(None), da.pat.FreePattern(None)])
PatternExpr_592 = da.pat.TuplePattern([da.pat.ConstantPattern('p1a'), da.pat.FreePattern('leader_scout'), da.pat.FreePattern('b')])
PatternExpr_615 = da.pat.TuplePattern([da.pat.ConstantPattern('p2a'), da.pat.FreePattern('leader_commander'), da.pat.FreePattern('b'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_504 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_689 = da.pat.TuplePattern([da.pat.ConstantPattern('p2b'), da.pat.FreePattern('a'), da.pat.BoundPattern('_BoundPattern694_')])
PatternExpr_724 = da.pat.TuplePattern([da.pat.ConstantPattern('p2b'), da.pat.FreePattern(None), da.pat.FreePattern('b1')])
PatternExpr_796 = da.pat.TuplePattern([da.pat.ConstantPattern('p1b'), da.pat.FreePattern('a'), da.pat.BoundPattern('_BoundPattern801_'), da.pat.FreePattern(None)])
PatternExpr_828 = da.pat.TuplePattern([da.pat.ConstantPattern('p1b'), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern832_'), da.pat.FreePattern('r')])
PatternExpr_866 = da.pat.TuplePattern([da.pat.ConstantPattern('p1b'), da.pat.FreePattern('a'), da.pat.FreePattern('b1'), da.pat.FreePattern(None)])
PatternExpr_956 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_976 = da.pat.TuplePattern([da.pat.ConstantPattern('propose'), da.pat.FreePattern('s'), da.pat.FreePattern('p')])
PatternExpr_1028 = da.pat.TuplePattern([da.pat.ConstantPattern('adopted'), da.pat.FreePattern('ballot_num'), da.pat.FreePattern('pvals')])
PatternExpr_1080 = da.pat.TuplePattern([da.pat.ConstantPattern('preempted'), da.pat.TuplePattern([da.pat.FreePattern('r1'), da.pat.FreePattern('leader1')])])
PatternExpr_961 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
PatternExpr_1269 = da.pat.TuplePattern([da.pat.ConstantPattern('response'), da.pat.FreePattern('cid'), da.pat.FreePattern('result')])
PatternExpr_1470 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_1475 = da.pat.BoundPattern('_BoundPattern1476_')
PatternExpr_1477 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern1483_')]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
_config_object = {}
import sys
import time
import random
NOPS = 10

def operation(i):
    return (lambda state: ((state + [i]), ['result', i, 'on', state]))
operations = {i: operation(i) for i in range(NOPS)}

class Replica(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._ReplicaReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_0', PatternExpr_382, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_1', PatternExpr_402, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Replica_handler_401]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ReplicaReceivedEvent_2', PatternExpr_419, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Replica_handler_418])])

    def setup(self, leaders, initial_state, **rest_1497):
        super().setup(leaders=leaders, initial_state=initial_state, **rest_1497)
        self._state.leaders = leaders
        self._state.initial_state = initial_state
        self._state.state = self._state.initial_state
        self._state.slot_num = 1
        self._state.proposals = set()
        self._state.decisions = set()

    def run(self):
        self.debug('### start')
        super()._label('_st_label_379', block=False)
        _st_label_379 = 0
        while (_st_label_379 == 0):
            _st_label_379 += 1
            if PatternExpr_387.match_iter(self._ReplicaReceivedEvent_0, SELF_ID=self._id):
                _st_label_379 += 1
            else:
                super()._label('_st_label_379', block=True)
                _st_label_379 -= 1
        self.output('terminating')

    def propose(self, p):

        def ExistentialOpExpr_243():
            for (_, _BoundPattern247_) in self._state.decisions:
                if (_BoundPattern247_ == p):
                    if True:
                        return True
            return False
        if (not ExistentialOpExpr_243()):
            maxs = max(({s for (s, _) in (self._state.proposals | self._state.decisions)} or {0}))

            def ExistentialOpExpr_285(s):
                for (_BoundPattern288_, _) in (self._state.proposals | self._state.decisions):
                    if (_BoundPattern288_ == s):
                        if True:
                            return True
                return False
            s1 = min({s for s in range(1, ((maxs + 1) + 1)) if (not ExistentialOpExpr_285(s=s))})
            self._state.proposals.add((s1, p))
            self.send(('propose', s1, p), self._state.leaders)

    def perform(self, p):
        self.debug('### perform', p)
        (client, cid, op) = p
        s = None

        def ExistentialOpExpr_328():
            nonlocal s
            for (s, _BoundPattern333_) in self._state.decisions:
                if (_BoundPattern333_ == p):
                    if (s < self._state.slot_num):
                        return True
            return False
        if ExistentialOpExpr_328():
            self._state.slot_num += 1
        else:
            self.debug('===', self._state.state, op)
            (next, result) = operations[op](self._state.state)
            self.debug('===', next, result)
            self._state.state = next
            self._state.slot_num += 1
            self.send(('response', cid, result), client)

    def _Replica_handler_401(self, p):
        self.debug('### request', p)
        self.propose(p)
    _Replica_handler_401._labels = None
    _Replica_handler_401._notlabels = None

    def _Replica_handler_418(self, s, p):
        self.debug('### decision', s, p)
        self._state.decisions.add((s, p))
        p1 = None

        def ExistentialOpExpr_441():
            nonlocal p1
            for (_BoundPattern444_, p1) in self._state.decisions:
                if (_BoundPattern444_ == self._state.slot_num):
                    if True:
                        return True
            return False
        while ExistentialOpExpr_441():
            p2 = None

            def ExistentialOpExpr_451():
                nonlocal p2
                for (_BoundPattern454_, p2) in self._state.proposals:
                    if (_BoundPattern454_ == self._state.slot_num):
                        if (not (p2 == p1)):
                            return True
                return False
            if ExistentialOpExpr_451():
                self.propose(p2)
            self.perform(p1)
    _Replica_handler_418._labels = None
    _Replica_handler_418._notlabels = None

class Acceptor(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._AcceptorReceivedEvent_0 = []
        self._AcceptorReceivedEvent_2 = []
        self._AcceptorReceivedEvent_3 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_0', PatternExpr_499, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_1', PatternExpr_519, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_518]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_2', PatternExpr_537, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_3', PatternExpr_561, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_4', PatternExpr_592, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_591]), da.pat.EventPattern(da.pat.ReceivedEvent, '_AcceptorReceivedEvent_5', PatternExpr_615, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Acceptor_handler_614])])

    def setup(self, **rest_1497):
        super().setup(**rest_1497)
        self._state.ballot_num = None
        self._state.accepted = set()

    def run(self):
        self.debug('### start')
        super()._label('_st_label_496', block=False)
        _st_label_496 = 0
        while (_st_label_496 == 0):
            _st_label_496 += 1
            if PatternExpr_504.match_iter(self._AcceptorReceivedEvent_0, SELF_ID=self._id):
                _st_label_496 += 1
            else:
                super()._label('_st_label_496', block=True)
                _st_label_496 -= 1
        self.output('terminating')

    def _Acceptor_handler_518(self, m):
        BOTTOM = ((- 1), (- 1))
        self._state.ballot_num = max((({b for (_, _, (_ConstantPattern553_, _, b)) in self._AcceptorReceivedEvent_2 if (_ConstantPattern553_ == 'p1a')} | {b for (_, _, (_ConstantPattern579_, _, b, _, _)) in self._AcceptorReceivedEvent_3 if (_ConstantPattern579_ == 'p2a')}) or {BOTTOM}))
    _Acceptor_handler_518._labels = None
    _Acceptor_handler_518._notlabels = None

    def _Acceptor_handler_591(self, leader_scout, b):
        self.debug('### p1a', leader_scout, b)
        self.send(('p1b', self._id, self._state.ballot_num, self._state.accepted), leader_scout)
    _Acceptor_handler_591._labels = None
    _Acceptor_handler_591._notlabels = None

    def _Acceptor_handler_614(self, leader_commander, b, s, p):
        self.debug('### p2a', leader_commander, b, s, p)
        if (b == self._state.ballot_num):
            self._state.accepted.add((b, s, p))
        self.send(('p2b', self._id, self._state.ballot_num), leader_commander)
    _Acceptor_handler_614._labels = None
    _Acceptor_handler_614._notlabels = None

class Commander(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._CommanderReceivedEvent_0 = []
        self._CommanderReceivedEvent_1 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_CommanderReceivedEvent_0', PatternExpr_689, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CommanderReceivedEvent_1', PatternExpr_724, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, leader, acceptors, replicas, b, s, p, **rest_1497):
        super().setup(leader=leader, acceptors=acceptors, replicas=replicas, b=b, s=s, p=p, **rest_1497)
        self._state.leader = leader
        self._state.acceptors = acceptors
        self._state.replicas = replicas
        self._state.b = b
        self._state.s = s
        self._state.p = p
        pass

    def run(self):
        self.debug('### start')
        self.send(('p2a', self._id, self._state.b, self._state.s, self._state.p), self._state.acceptors)
        super()._label('_st_label_684', block=False)
        b1 = None

        def ExistentialOpExpr_722():
            nonlocal b1
            for (_, _, (_ConstantPattern740_, _, b1)) in self._CommanderReceivedEvent_1:
                if (_ConstantPattern740_ == 'p2b'):
                    if (not (b1 == self._state.b)):
                        return True
            return False
        _st_label_684 = 0
        while (_st_label_684 == 0):
            _st_label_684 += 1
            if (len({a for (_, _, (_ConstantPattern705_, a, _BoundPattern708_)) in self._CommanderReceivedEvent_0 if (_ConstantPattern705_ == 'p2b') if (_BoundPattern708_ == self._state.b)}) > (len(self._state.acceptors) / 2)):
                self.send(('decision', self._state.s, self._state.p), self._state.replicas)
                _st_label_684 += 1
            elif ExistentialOpExpr_722():
                self.send(('preempted', b1), self._state.leader)
                _st_label_684 += 1
            else:
                super()._label('_st_label_684', block=True)
                _st_label_684 -= 1

class Scout(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._ScoutReceivedEvent_0 = []
        self._ScoutReceivedEvent_1 = []
        self._ScoutReceivedEvent_2 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ScoutReceivedEvent_0', PatternExpr_796, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ScoutReceivedEvent_1', PatternExpr_828, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_ScoutReceivedEvent_2', PatternExpr_866, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, leader, acceptors, b, **rest_1497):
        super().setup(leader=leader, acceptors=acceptors, b=b, **rest_1497)
        self._state.leader = leader
        self._state.acceptors = acceptors
        self._state.b = b
        self._state.pvalues = set()

    def run(self):
        self.debug('### start')
        time.sleep(random.random())
        self.send(('p1a', self._id, self._state.b), self._state.acceptors)
        super()._label('_st_label_791', block=False)
        b1 = a = None

        def ExistentialOpExpr_864():
            nonlocal b1, a
            for (_, _, (_ConstantPattern884_, a, b1, _)) in self._ScoutReceivedEvent_2:
                if (_ConstantPattern884_ == 'p1b'):
                    if (not (b1 == self._state.b)):
                        return True
            return False
        _st_label_791 = 0
        while (_st_label_791 == 0):
            _st_label_791 += 1
            if (len({a for (_, _, (_ConstantPattern813_, a, _BoundPattern816_, _)) in self._ScoutReceivedEvent_0 if (_ConstantPattern813_ == 'p1b') if (_BoundPattern816_ == self._state.b)}) > (len(self._state.acceptors) / 2)):
                self._state.pvalues = {v for (_, _, (_ConstantPattern845_, _, _BoundPattern848_, r)) in self._ScoutReceivedEvent_1 if (_ConstantPattern845_ == 'p1b') if (_BoundPattern848_ == self._state.b) for v in r}
                self.send(('adopted', self._state.b, self._state.pvalues), self._state.leader)
                _st_label_791 += 1
            elif ExistentialOpExpr_864():
                self.send(('preempted', b1), self._state.leader)
                _st_label_791 += 1
            else:
                super()._label('_st_label_791', block=True)
                _st_label_791 -= 1

class Leader(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._LeaderReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_0', PatternExpr_956, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_1', PatternExpr_976, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Leader_handler_975]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_2', PatternExpr_1028, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Leader_handler_1027]), da.pat.EventPattern(da.pat.ReceivedEvent, '_LeaderReceivedEvent_3', PatternExpr_1080, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Leader_handler_1079])])

    def setup(self, acceptors, replicas, **rest_1497):
        super().setup(acceptors=acceptors, replicas=replicas, **rest_1497)
        self._state.acceptors = acceptors
        self._state.replicas = replicas
        self._state.ballot_num = (0, self._id)
        self._state.active = False
        self._state.proposals = set()

    def run(self):
        self.debug('### start')
        sub = self.new(Scout, (self._id, self._state.acceptors, self._state.ballot_num), method='thread', daemon=True)
        self._start(sub)
        super()._label('_st_label_953', block=False)
        _st_label_953 = 0
        while (_st_label_953 == 0):
            _st_label_953 += 1
            if PatternExpr_961.match_iter(self._LeaderReceivedEvent_0, SELF_ID=self._id):
                _st_label_953 += 1
            else:
                super()._label('_st_label_953', block=True)
                _st_label_953 -= 1
        self.output('terminating')

    def circle_plus(self, x, y):

        def ExistentialOpExpr_1137(s):
            for (_BoundPattern1140_, _) in y:
                if (_BoundPattern1140_ == s):
                    if True:
                        return True
            return False
        return (y | {(s, p) for (s, p) in x if (not ExistentialOpExpr_1137(s=s))})

    def pmax(self, pvals):

        def UniversalOpExpr_1163(s, b):
            for (b1, _BoundPattern1168_, _) in pvals:
                if (_BoundPattern1168_ == s):
                    if (not (b1 <= b)):
                        return False
            return True
        return {(s, p) for (b, s, p) in pvals if UniversalOpExpr_1163(s=s, b=b)}

    def _Leader_handler_975(self, s, p):
        self.debug('### propose', s, p)

        def ExistentialOpExpr_992():
            for (_BoundPattern995_, _) in self._state.proposals:
                if (_BoundPattern995_ == s):
                    if True:
                        return True
            return False
        if (not ExistentialOpExpr_992()):
            self._state.proposals.add((s, p))
            if self._state.active:
                sub = self.new(Commander, (self._id, self._state.acceptors, self._state.replicas, self._state.ballot_num, s, p), method='thread', daemon=True)
                self._start(sub)
    _Leader_handler_975._labels = None
    _Leader_handler_975._notlabels = None

    def _Leader_handler_1027(self, ballot_num, pvals):
        self.debug('### adopted', ballot_num, pvals)
        self._state.proposals = self.circle_plus(self._state.proposals, self.pmax(pvals))
        for (s, p) in self._state.proposals:
            sub = self.new(Commander, (self._id, self._state.acceptors, self._state.replicas, ballot_num, s, p), method='thread', daemon=True)
            self._start(sub)
        self._state.active = True
    _Leader_handler_1027._labels = None
    _Leader_handler_1027._notlabels = None

    def _Leader_handler_1079(self, r1, leader1):
        if ((r1, leader1) > self._state.ballot_num):
            self._state.active = False
            self._state.ballot_num = ((r1 + 1), self._id)
            sub = self.new(Scout, (self._id, self._state.acceptors, self._state.ballot_num), method='thread', daemon=True)
            self._start(sub)
    _Leader_handler_1079._labels = None
    _Leader_handler_1079._notlabels = None

class Client(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_ClientReceivedEvent_0', PatternExpr_1269, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Client_handler_1268])])

    def setup(self, replicas, nops, **rest_1497):
        super().setup(replicas=replicas, nops=nops, **rest_1497)
        self._state.replicas = replicas
        self._state.nops = nops
        self._state.cid = 0
        self._state.results = dict()
        self._state.count = dict()

    def run(self):
        for i in range(self._state.nops):
            self.send(('request', (self._id, self._state.cid, random.randint(0, (NOPS - 1)))), self._state.replicas)
            super()._label('_st_label_1232', block=False)
            _st_label_1232 = 0
            while (_st_label_1232 == 0):
                _st_label_1232 += 1
                if (self._state.cid in self._state.results):
                    _st_label_1232 += 1
                else:
                    super()._label('_st_label_1232', block=True)
                    _st_label_1232 -= 1
            else:
                if (_st_label_1232 != 2):
                    continue
            if (_st_label_1232 != 2):
                break
            self.output('received result', self._state.cid, self._state.results[self._state.cid])
            self._state.cid += 1
        super()._label('_st_label_1246', block=False)
        cid = None

        def UniversalOpExpr_1247():
            nonlocal cid
            for self._state.cid in range(self._state.nops):
                if (not (self._state.count[self._state.cid] == len(self._state.replicas))):
                    return False
            return True
        _st_label_1246 = 0
        while (_st_label_1246 == 0):
            _st_label_1246 += 1
            if UniversalOpExpr_1247():
                _st_label_1246 += 1
            else:
                super()._label('_st_label_1246', block=True)
                _st_label_1246 -= 1
        self.output('terminating')
        self.send(('done',), self.parent())

    def _Client_handler_1268(self, cid, result):
        self.debug('### response', cid, result)
        if (not (cid in self._state.results)):
            self._state.results[cid] = result
        elif (not (self._state.results[cid] == result)):
            self.error('different result', cid, result, 'than', self._state.results[cid])
        self._state.count[cid] = (1 if (not (cid in self._state.count)) else (self._state.count[cid] + 1))
    _Client_handler_1268._labels = None
    _Client_handler_1268._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._Node_ReceivedEvent_0 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_Node_ReceivedEvent_0', PatternExpr_1470, sources=[PatternExpr_1475], destinations=None, timestamps=None, record_history=True, handlers=[])])

    def run(self):
        nacceptors = (int(sys.argv[1]) if (len(sys.argv) > 1) else 5)
        nreplicas = (int(sys.argv[2]) if (len(sys.argv) > 2) else 4)
        nleaders = (int(sys.argv[3]) if (len(sys.argv) > 3) else 2)
        nclients = (int(sys.argv[4]) if (len(sys.argv) > 4) else 3)
        nops = (int(sys.argv[5]) if (len(sys.argv) > 5) else 3)
        acceptors = self.new(Acceptor, (), num=nacceptors)
        replicas = self.new(Replica, num=nreplicas)
        leaders = self.new(Leader, (acceptors, replicas), num=nleaders)
        initial_state = []
        self._setup(replicas, (leaders, initial_state))
        clients = self.new(Client, (replicas, nops), num=nclients)
        self._start(acceptors)
        self._start((replicas | leaders))
        self._start(clients)
        super()._label('_st_label_1461', block=False)
        c = None

        def UniversalOpExpr_1462():
            nonlocal c
            for c in clients:
                if (not PatternExpr_1477.match_iter(self._Node_ReceivedEvent_0, _BoundPattern1483_=c)):
                    return False
            return True
        _st_label_1461 = 0
        while (_st_label_1461 == 0):
            _st_label_1461 += 1
            if UniversalOpExpr_1462():
                _st_label_1461 += 1
            else:
                super()._label('_st_label_1461', block=True)
                _st_label_1461 -= 1
        self.output('All clients done.')
        self.send(('done',), ((acceptors | replicas) | leaders))



# -*- generated by 1.0.2 -*-
import da
PatternExpr_190 = da.pat.TuplePattern([da.pat.ConstantPattern('vote'), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern194_')])
PatternExpr_197 = da.pat.BoundPattern('_BoundPattern198_')
PatternExpr_222 = da.pat.TuplePattern([da.pat.ConstantPattern('vote'), da.pat.ConstantPattern('abort'), da.pat.BoundPattern('_BoundPattern227_')])
PatternExpr_230 = da.pat.BoundPattern('_BoundPattern231_')
PatternExpr_252 = da.pat.TuplePattern([da.pat.ConstantPattern('vote'), da.pat.ConstantPattern('ready'), da.pat.BoundPattern('_BoundPattern257_')])
PatternExpr_260 = da.pat.FreePattern('c')
PatternExpr_296 = da.pat.TuplePattern([da.pat.ConstantPattern('vote'), da.pat.ConstantPattern('ready'), da.pat.BoundPattern('_BoundPattern301_')])
PatternExpr_304 = da.pat.BoundPattern('_BoundPattern305_')
PatternExpr_332 = da.pat.TuplePattern([da.pat.ConstantPattern('done'), da.pat.BoundPattern('_BoundPattern335_')])
PatternExpr_338 = da.pat.BoundPattern('_BoundPattern339_')
PatternExpr_232 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern238_')]), da.pat.TuplePattern([da.pat.ConstantPattern('vote'), da.pat.ConstantPattern('abort'), da.pat.BoundPattern('_BoundPattern244_')])])
PatternExpr_306 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern312_')]), da.pat.TuplePattern([da.pat.ConstantPattern('vote'), da.pat.ConstantPattern('ready'), da.pat.BoundPattern('_BoundPattern318_')])])
PatternExpr_340 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.BoundPattern('_BoundPattern346_')]), da.pat.TuplePattern([da.pat.ConstantPattern('done'), da.pat.BoundPattern('_BoundPattern350_')])])
PatternExpr_401 = da.pat.TuplePattern([da.pat.ConstantPattern('prepare'), da.pat.FreePattern('tid')])
PatternExpr_408 = da.pat.FreePattern('coord')
PatternExpr_437 = da.pat.TuplePattern([da.pat.ConstantPattern('abort'), da.pat.FreePattern('tid')])
PatternExpr_450 = da.pat.TuplePattern([da.pat.ConstantPattern('commit'), da.pat.FreePattern('tid')])
PatternExpr_457 = da.pat.FreePattern('coord')
PatternExpr_511 = da.pat.TuplePattern([da.pat.ConstantPattern('done')])
PatternExpr_516 = da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.TuplePattern([da.pat.FreePattern(None), da.pat.FreePattern(None), da.pat.FreePattern(None)]), da.pat.TuplePattern([da.pat.ConstantPattern('done')])])
_config_object = {}
import sys
from random import randint

class Coordinator(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._CoordinatorReceivedEvent_0 = []
        self._CoordinatorReceivedEvent_1 = []
        self._CoordinatorReceivedEvent_2 = []
        self._CoordinatorReceivedEvent_3 = []
        self._CoordinatorReceivedEvent_4 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_CoordinatorReceivedEvent_0', PatternExpr_190, sources=[PatternExpr_197], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CoordinatorReceivedEvent_1', PatternExpr_222, sources=[PatternExpr_230], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CoordinatorReceivedEvent_2', PatternExpr_252, sources=[PatternExpr_260], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CoordinatorReceivedEvent_3', PatternExpr_296, sources=[PatternExpr_304], destinations=None, timestamps=None, record_history=True, handlers=[]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CoordinatorReceivedEvent_4', PatternExpr_332, sources=[PatternExpr_338], destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, tid, cohorts, **rest_585):
        super().setup(tid=tid, cohorts=cohorts, **rest_585)
        self._state.tid = tid
        self._state.cohorts = cohorts
        pass

    def run(self):
        self.to_commit()
        self.send(('done',), self._state.cohorts)
        self.output('terminating')

    def to_commit(self):
        self.send(('prepare', self._state.tid), self._state.cohorts)
        super()._label('_st_label_181', block=False)
        c = None

        def UniversalOpExpr_182():
            nonlocal c
            for c in self._state.cohorts:

                def ExistentialOpExpr_188(c):
                    for (_, (_, _, _BoundPattern205_), (_ConstantPattern207_, _, _BoundPattern210_)) in self._CoordinatorReceivedEvent_0:
                        if (_BoundPattern205_ == c):
                            if (_ConstantPattern207_ == 'vote'):
                                if (_BoundPattern210_ == self._state.tid):
                                    if True:
                                        return True
                    return False
                if (not ExistentialOpExpr_188(c=c)):
                    return False
            return True
        _st_label_181 = 0
        while (_st_label_181 == 0):
            _st_label_181 += 1
            if UniversalOpExpr_182():
                _st_label_181 += 1
            else:
                super()._label('_st_label_181', block=True)
                _st_label_181 -= 1
        c = None

        def ExistentialOpExpr_215():
            nonlocal c
            for c in self._state.cohorts:
                if PatternExpr_232.match_iter(self._CoordinatorReceivedEvent_1, _BoundPattern238_=c, _BoundPattern244_=self._state.tid, SELF_ID=self._id):
                    return True
            return False
        if ExistentialOpExpr_215():
            s = {c for c in self._state.cohorts for (_, (_, _, _FreePattern268_), (_ConstantPattern270_, _ConstantPattern272_, _BoundPattern274_)) in self._CoordinatorReceivedEvent_2 if (_FreePattern268_ == c) if (_ConstantPattern270_ == 'vote') if (_ConstantPattern272_ == 'ready') if (_BoundPattern274_ == self._state.tid)}
            self.send(('abort', self._state.tid), s)
            self.abort(self._state.tid)
        c = None

        def UniversalOpExpr_289():
            nonlocal c
            for c in self._state.cohorts:
                if (not PatternExpr_306.match_iter(self._CoordinatorReceivedEvent_3, _BoundPattern312_=c, _BoundPattern318_=self._state.tid, SELF_ID=self._id)):
                    return False
            return True
        if UniversalOpExpr_289():
            self.send(('commit', self._state.tid), self._state.cohorts)
            super()._label('_st_label_324', block=False)
            c = None

            def UniversalOpExpr_325():
                nonlocal c
                for c in self._state.cohorts:
                    if (not PatternExpr_340.match_iter(self._CoordinatorReceivedEvent_4, _BoundPattern346_=c, _BoundPattern350_=self._state.tid, SELF_ID=self._id)):
                        return False
                return True
            _st_label_324 = 0
            while (_st_label_324 == 0):
                _st_label_324 += 1
                if UniversalOpExpr_325():
                    _st_label_324 += 1
                else:
                    super()._label('_st_label_324', block=True)
                    _st_label_324 -= 1
            self.commit(self._state.tid)

    def abort(self, tid):
        self.output(('abort:' + str(tid)))

    def commit(self, tid):
        self.output(('commit:' + str(tid)))

class Cohort(da.DistProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._CohortReceivedEvent_3 = []
        self._events.extend([da.pat.EventPattern(da.pat.ReceivedEvent, '_CohortReceivedEvent_0', PatternExpr_401, sources=[PatternExpr_408], destinations=None, timestamps=None, record_history=None, handlers=[self._Cohort_handler_400]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CohortReceivedEvent_1', PatternExpr_437, sources=None, destinations=None, timestamps=None, record_history=None, handlers=[self._Cohort_handler_436]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CohortReceivedEvent_2', PatternExpr_450, sources=[PatternExpr_457], destinations=None, timestamps=None, record_history=None, handlers=[self._Cohort_handler_449]), da.pat.EventPattern(da.pat.ReceivedEvent, '_CohortReceivedEvent_3', PatternExpr_511, sources=None, destinations=None, timestamps=None, record_history=True, handlers=[])])

    def setup(self, failure_rate, **rest_585):
        super().setup(failure_rate=failure_rate, **rest_585)
        self._state.failure_rate = failure_rate
        pass

    def run(self):
        super()._label('_st_label_508', block=False)
        _st_label_508 = 0
        while (_st_label_508 == 0):
            _st_label_508 += 1
            if PatternExpr_516.match_iter(self._CohortReceivedEvent_3, SELF_ID=self._id):
                _st_label_508 += 1
            else:
                super()._label('_st_label_508', block=True)
                _st_label_508 -= 1

    def prepared(self, tid):
        return (randint(0, 100) > self._state.failure_rate)

    def ready(self, tid):
        self.output(('ready:' + str(tid)))

    def abort(self, tid):
        self.output(('abort:' + str(tid)))

    def commit(self, tid):
        self.output(('commit:' + str(tid)))

    def _Cohort_handler_400(self, tid, coord):
        if self.prepared(tid):
            self.send(('vote', 'ready', tid), coord)
            self.ready(tid)
        else:
            self.send(('vote', 'abort', tid), coord)
            self.abort(tid)
    _Cohort_handler_400._labels = None
    _Cohort_handler_400._notlabels = None

    def _Cohort_handler_436(self, tid):
        self.abort(tid)
    _Cohort_handler_436._labels = None
    _Cohort_handler_436._notlabels = None

    def _Cohort_handler_449(self, tid, coord):
        self.send(('done', tid), coord)
        self.commit(tid)
    _Cohort_handler_449._labels = None
    _Cohort_handler_449._notlabels = None

class Node_(da.NodeProcess):

    def __init__(self, procimpl, props):
        super().__init__(procimpl, props)
        self._events.extend([])

    def run(self):
        ncohorts = (int(sys.argv[1]) if (len(sys.argv) > 1) else 10)
        fail_rate = (int(sys.argv[2]) if (len(sys.argv) > 2) else 10)
        cohorts = self.new(Cohort, (fail_rate,), num=ncohorts)
        coordinators = self.new(Coordinator, ('001', cohorts), num=1)
        self._start((cohorts | coordinators))
